#!/usr/bin/perl

######################################################################
# Production d'Intellique.
# Date : 31 janvier 2005.
# Utilisation : Affichage html du systeme et des cartes 3ware.
# Auteur : Sylvain Pujos
######################################################################

do '../web-lib.pl';
&init_config();

use Sys::Hostname;

use lib './modules';    # dossier des modules specifiques
use html;               # modules specifiques pour les pages html

my $got_fonctions;

BEGIN {
    if ( $got_fonctions = eval { require Storiq::Infosystem } )
    {                   # module contenant les fonctions
        Infosystem->import(qw( infosysteme ));
    }
}

#--------------Affichage html--------------------------
sub infosystem () {

    #--------------Debut--------------------------
    print_debut( $text{'index_title'}, hostname() );

    if ($got_fonctions) {
        my %Infosysteme = infosysteme();

        #--------------Système--------------------------
        print_box_debut( $text{'system'} );
        print_ligne_simple( $text{'cano_host'}, $Infosysteme{"hostname"} );
        foreach my $interface ( sort keys( %{ $Infosysteme{"interfaces"} } ) ) {
            print_ligne_simple( "IP : $interface",
                $Infosysteme{"interfaces"}{"$interface"}{"Ip"} );
        }
        print_ligne_simple( $text{'kern_vers'},  $Infosysteme{"kernel"} );
        print_ligne_simple( $text{'uptime'},     $Infosysteme{"uptime"} );
        print_ligne_simple( $text{'sys_charge'}, $Infosysteme{"loadavg"} );

        print_box_fin();
        print "   <br>";

        #--------------Réseau--------------------------
        print_box_debut( $text{'network'} );

        print_ligne_reseau(
            "<b>$text{'device'}</b>", "<b>$text{'down'}</b>",
            "<b>$text{'up'}</b>",     "<b>$text{'err'}</b>",
            "<b>$text{'drop'}</b>"
        );
        foreach my $interface ( sort keys( %{ $Infosysteme{"interfaces"} } ) ) {
            print_ligne_reseau(
                $interface,
                $Infosysteme{"interfaces"}{$interface}{"Recoi"},
                $Infosysteme{"interfaces"}{$interface}{"Envoi"},
                $Infosysteme{"interfaces"}{$interface}{"Erreur"},
                $Infosysteme{"interfaces"}{$interface}{"Drop"}
            );
        }

        print_box_fin();
        print '  </td><td valign="top" width="50%">';

        #--------------Information Matériel--------------------------
        print_box_debut( $text{'info_matos'} );

        print_ligne_simple(
            "$text{'proc'}&nbsp;&nbsp;&nbsp;",
            $Infosysteme{"cpuinfo"}{"Processeur"}
        );
        print_ligne_simple( "$text{'modele'}",
            $Infosysteme{"cpuinfo"}{"Modèle"} );
        print_ligne_simple( "$text{'freq'}",
            $Infosysteme{"cpuinfo"}{"Fréquence"} );
        print_ligne_simple( "$text{'cache_size'}",
            $Infosysteme{"cpuinfo"}{"Taille Cache"} );
        print_ligne_simple( "$text{'bogo'}",
            $Infosysteme{"cpuinfo"}{"Bogomips"} );

        for my $peripherique ( "PCI", "IDE", "SCSI", "USB" ) {
            if ( $Infosysteme{$peripherique} ) {
                my @tab;
                foreach
                  my $line ( sort keys( %{ $Infosysteme{$peripherique} } ) )
                {
                    push( @tab, $Infosysteme{$peripherique}{$line} );
                }
                print_ligne_simple( "$text{'device'}. $peripherique",
                    join( "<br>", @tab ) );
            }
        }

        print_box_fin();
        print '  </td></tr><tr><td colspan="2">';

        #--------------Utilisation mémoire--------------------------
        my %purcent = barre_color_mem(
            $Infosysteme{"meminfo"}{"MemPurcent"},
            $Infosysteme{"meminfo"}{"SwapPurcent"}
        );

        print_box_debut( $text{'mem_use'} );

        print_ligne_memoire(
            "<b>$text{'type'}</b>",  "<b>$text{'utilisation'}</b>",
            "<b>$text{'libre'}</b>", "<b>$text{'utilise'}</b>",
            "<b>$text{'taille'}</b>"
        );
        print_ligne_memoire(
            $text{'phys_mem'},
            $purcent{"Mem"},
            $Infosysteme{"meminfo"}{"MemFree"},
            $Infosysteme{"meminfo"}{"MemUsed"},
            $Infosysteme{"meminfo"}{"MemTotal"}
        );
        print_ligne_memoire(
            $text{'swap'},
            $purcent{"Swap"},
            $Infosysteme{"meminfo"}{"SwapFree"},
            $Infosysteme{"meminfo"}{"SwapUsed"},
            $Infosysteme{"meminfo"}{"SwapTotal"}
        );

        print_box_fin();
        print '  </td></tr><tr><td colspan="2">';

        #--------------Systèmes de fichiers montés--------------------------
        print_box_debut( $text{'mount_fs'} );

        print_ligne_filesystem(
            "<b>$text{'point'}</b>", "<b>$text{'type'}</b>",
            "<b>$text{'part'}</b>",  "<b>$text{'utilisation'}</b>",
            "<b>$text{'libre'}</b>", "<b>$text{'utilise'}</b>",
            "<b>$text{'taille'}</b>"
        );

        foreach
          my $filesystems ( sort keys( %{ $Infosysteme{"filesystems"} } ) )
        {
            if ( $filesystems ne "total" ) {
                print_ligne_filesystem(
                    $filesystems,
                    $Infosysteme{"filesystems"}{$filesystems}{"Type"},
                    $Infosysteme{"filesystems"}{$filesystems}{"Partition"},
                    barre_color_fs(
                        $Infosysteme{"filesystems"}{$filesystems}{"Purcent"}
                    ),
                    $Infosysteme{"filesystems"}{$filesystems}{"Free"},
                    $Infosysteme{"filesystems"}{$filesystems}{"Used"},
                    $Infosysteme{"filesystems"}{$filesystems}{"Size"}
                );
            }
        }

        print_ligne_filesystem_totaux(
            barre_color_fs( $Infosysteme{'filesystems'}{'total'}{'Purcent'} ),
            $Infosysteme{'filesystems'}{'total'}{'Free'},
            $Infosysteme{'filesystems'}{'total'}{'Used'},
            $Infosysteme{'filesystems'}{'total'}{'Size'}
        );
        print_box_fin();
        print '  </td></tr>';

        if ( $Infosysteme{raid} ) {

            print_box_debut( $text{'raid_ctl'} );

            print_ligne_controleur(
                "<b>$text{'ctler'}"
                  . espace(8)
                  . $text{'unit'}
                  . espace(8)
                  . "$text{'port'}</b>",
                "<b>$text{'type'}</b>",
                "<b>$text{'modele'}</b>",
                "<b>$text{'etat'}</b>",
                "<b>$text{'cmpl'}</b>",
                "<b>$text{'taille'}</b>"
            );

            foreach my $controleur ( sort keys( %{ $Infosysteme{raid} } ) ) {
                my $status = $text{'optimal'};
                $status = $text{'noptimal'}
                  if ( $Infosysteme{raid}{$controleur}{status} != 0 );

                print_ligne_controleur(
                    espace(6) . $controleur,
                    "", $Infosysteme{raid}{$controleur}{model},
                    $status, "", ""
                );

                foreach my $unite (
                    sort keys %{ $Infosysteme{raid}{$controleur}{arrays} } )
                {
                    my $cmp = "---";
                    $cmp = "---"
                      if ( $Infosysteme{raid}{$controleur}{arrays}{$unite}
                        {progression} ne "-" );

                    my $status = 'OK';
                    if ( $Infosysteme{raid}{$controleur}{arrays}{$unite}
                        {status} != 0 )
                    {
                        $status = "NOT OK";
                    }

                    print_ligne_controleur(
                        espace(27) . $unite,
                        "RAID-"
                          . $Infosysteme{raid}{$controleur}{arrays}{$unite}
                          {raidtype},
                        "", $status, $cmp,
                        gbytesize(
                            $Infosysteme{raid}{$controleur}{arrays}{$unite}
                              {size}
                        )
                    );

                }
            }

        } else {

            print_box_debut( $text{'err_modul_func'} );
            print_box_fin();

        }

        print_box_fin();
    } else {
		print "missing library package: storiq-lib-infosystem";
	}

    #--------------Fin--------------------------
    print_fin();
}

sub barre_color_mem () {
    my $purcent_mem  = shift;
    my $purcent_swap = shift;
    my $color_swap;
    my $color_mem;
    chop $purcent_swap;
    chop $purcent_mem;
    my %retour;

    if ( $purcent_swap > 10 ) {
        $color_swap = "red";
        $color_mem  = "red";
    } elsif ( $purcent_mem > 90 ) {
        $color_swap = "green";
        $color_mem  = "orange";
    } else {
        $color_swap = "green";
        $color_mem  = "green";
    }

    $retour{"Mem"} =
        "<img src=\"images/bar_$color_mem.gif\" height=\"10\" width=\""
      . ( 2 * $purcent_mem + 2 )
      . "\" alt=\"\">&nbsp;&nbsp;$purcent_mem%";
    $retour{"Swap"} =
        "<img src=\"images/bar_$color_swap.gif\" height=\"10\" width=\""
      . ( 2 * $purcent_swap + 2 )
      . "\" alt=\"\">&nbsp;&nbsp;$purcent_swap%";

    return %retour;
}

sub barre_color_fs () {
    my $purcent = shift;
    my $color;
    chop $purcent;

    if ( $purcent < 80 ) {
        $color = "green";
    } elsif ( $purcent > 90 ) {
        $color = "red";
    } else {
        $color = "orange";
    }

    return
        "<img src=\"images/bar_$color.gif\" height=\"10\" width=\""
      . ( 2 * $purcent + 2 )
      . "\" alt=\"\">&nbsp;&nbsp;$purcent%";
}

sub barre_color () {
    my $purcent = shift;
    my $color   = shift;

    return
        "<img src=\"images/bar_$color.gif\" height=\"10\" width=\""
      . ( 2 * $purcent + 2 )
      . "\" alt=\"\">&nbsp;&nbsp;$purcent%";
}

sub espace () {
    my $nb_espace = shift;
    return "&nbsp;" x $nb_espace;
}

sub gbytesize () {
    my $size = shift;
    my $result;
    if ( $size > 1048576 ) {
        $result = ( sprintf "%.2f", ( $size / 1048576 ) ) . " To";
    } elsif ( $size > 1024 ) {
        $result = ( sprintf "%.2f", ( $size / 1024 ) ) . " Go";
    } else {
        $result = ( sprintf "%.2f", $size ) . " Mo";
    }

    return $result;
}

1;
