#!/usr/bin/perl

use Data::Dumper;

# require '../web-lib.pl';
# &init_config();

require Lib_Network;
require Objet_Logger;
require Objet_Bonding;

my $objet;

# Fonction qui cre un objet bonding
sub new_objet_bond {
    #################################################
    ## Creation de l objet bonding
    my $err;
    ($err, $objet) = new Objet_Logger( $config{path_log} );
    if ( $err ) {
        print "Error : Unable to instantiate Logger Object : $objet";
        &footer("/");
        exit;
    }

    my ($err2, $OBJ_BOND) = new Objet_Bonding($objet);
    if ( $err2 ) {
        print_and_exit($objet, "Unable to instantiate Bonding Object : $OBJ_BOND");
    }

    return $OBJ_BOND;
}


# Fonction qui affiche l'actuelle configuration des bondings
sub status {
    my $OBJ_BOND = new_objet_bond();
    my ($err, $status) = $OBJ_BOND->status();
    print_and_exit($status) if ($err);

    return %{ $status };
}


# Fonction de listing des interfaces libres..
# Fonction non documentee utilisee uniquement par la completion..
sub list_free_iface {
    my $hash = Lib_Network::get_ifaces_info();
    my @list;

    my %bond = map { $_ => undef } list_bond();

    foreach my $iface (keys(%$hash)) {
        # Interface est un bond
        next if ( exists $bond{ $iface } );

        # No virtual iface
        next if ( $iface =~ m/^.*:.*$/ );
        # Pas d interface sans mac adresse
        next if ( ! defined( $hash->{$iface}->{mac_addr} ) );

        # Pas en slave
        if ( ! defined( $hash->{$iface}->{ifconfig}->{slave} ) ) {
            push @list, $iface;
            next;
        }

        # Pas en slave        
        if ( !$hash->{$iface}->{ifconfig}->{slave} ) {
            push @list, $iface;
            next;
        }
    }
    return @list;
}

# Fonction de listing des bond en place
sub list_bond {
    my $OBJ_BOND = new_objet_bond();
    my ($err, $masters) = $OBJ_BOND->get_masters();
    print_and_exit($masters) if ($err);

    return @{ $masters };
}



#################################################
# Nouveau bonding
sub create_bond {
    my $OBJ_BOND = new_objet_bond();
    return $OBJ_BOND->create( @_ );
}

# Suprrime bonding
sub delete_bond {
    my $OBJ_BOND = new_objet_bond();
    return $OBJ_BOND->delete( @_ );
}

# Add interface
sub add_if {
    my $OBJ_BOND = new_objet_bond();
    return $OBJ_BOND->add_if( @_ );
}

# Del interface
sub del_if {
    my $OBJ_BOND = new_objet_bond();
    return $OBJ_BOND->del_if( @_ );
}

# Change le status du bond
sub bond_state {
    my $OBJ_BOND = new_objet_bond();
    my $bond    = shift;
    my $status  = shift;

    if ( $status eq "up" ) {
        my ($err, $etat) = $OBJ_BOND->start( $bond );
        Lib_Network::ifup( $bond );
	    return ($err, $etat);
	}

    my ($err, $etat) = $OBJ_BOND->stop( $bond );
    Lib_Network::ifdown( $bond );
    return ($err, $etat);
}

# Change le mode du bond
sub set_mode {
    my $OBJ_BOND = new_objet_bond();
    return $OBJ_BOND->set_mode( @_ );
}


# Fonction d'affichage d'une erreur, sort avec 1
# en valeur de retour
sub print_and_exit {
    my $error = shift;
    $objet->error("Bonding : $error");
    print "Error : ".$error."\n";
    
    
    exit 1;
}


#################################################
##Impression directement tout de suite (c joli)

sub print_code {
    print <<EOF;
<xmp>
    $_[0]
</xmp>
EOF
}

1;
