#!/usr/bin/perl -w
## ######### PROJECT NAME : ##########
## 
## Lib_Iscsi.pm for Objet_Partage
## 
## ######### PROJECT DESCRIPTION : ###
## 
## This file brings ietd iscsi support.
## Parser functions comes from Emmanuel Florac's
## iscsi-lib.pl; I made some modifications and
## correct one bug
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
## 
## Started on  Fri Mar 20 12:09:38 2009 Boutonnet Alexandre
## Last update Fri Mar 20 13:19:26 2009 Boutonnet Alexandre
## 
## ###################################
##

package Lib_Iscsi;

use strict;
use warnings;

# usefull for debug
use Data::Dumper;

my $IETD_CONF_FILE = "/etc/iet/ietd.conf";
my $GLOBAL_SECTION_NAME = "GLOBAL";


# ietd keywords
my @keywords=qw( Lun Alias IncomingUser OutgoingUser MaxConnections InitialR2T ImmediateData MaxRecvDataSegmentLength  MaxXmitDataSegmentLength MaxBurstLength FirstBurstLength DefaultTime2Wait DefaultTime2Retain MaxOutstandingR2T DataPDUInOrder DataSequenceInOrder ErrorRecoveryLevel HeaderDigest DataDigest Wthreads iSNSServer iSNSAccessControl QueuedCommands );


# Fonction de recuperation de toute la configuration
# Cette fonction ne prend rien en parametre
# et retour le traditionnel tableau (0, $hash) -> ok
# (!=0, $err_msg) -> fail
sub get_all
{
    my $hash = {};

    return (_parse_ietd_conf());
}

# Fonction de creation/modification d'une configuration d'un partage
# en fonction de la hash passee en param
# La hash donnee doit IMPERATIVEMENT posseder un id valide
# et retour le traditionnel tableau (0, 0) -> ok
# (!=0, $err_msg) -> fail
sub set
{
    my $hash = shift;

    return (1, "Invalid hash") if (!defined($hash));

    # Je recup la conf courante
    my ($err_code, $current_hash) = _parse_ietd_conf();
    return ($err_code, $current_hash) if ($err_code);

    # Je compare les deux confs..
    foreach my $new_lun ( sort keys(%{$hash}))
    {
	# C'est une modification
	if (exists($current_hash->{$new_lun}))
	{
	    foreach my $key (keys(%{$hash->{$new_lun}}))
	    {
		$current_hash->{$new_lun}->{$key} = $hash->{$new_lun}->{$key};
	    }
	}
	else # c'est un ajout
	{
	    $current_hash->{$new_lun} = $hash->{$new_lun};
	}
    }

    return (_write_ietd_conf($current_hash));
}

# Fonction de suppression d'une entree
# Cette fonction prend en parametre l'id de la conf
# a supprimer
# et retour le traditionnel tableau (0, 0) -> ok
# (!=0, $err_msg) -> fail
sub delete
{
    my $id = shift;

    return (1, "Invalid id") if (!defined($id));

    # Je recup la conf courante
    my ($err_code, $current_hash) = _parse_ietd_conf();
    return ($err_code, $current_hash) if ($err_code);

    delete $current_hash->{$id} if (exists($current_hash->{$id}));

    return (_write_ietd_conf($current_hash));
}


##### PARSING FILE (PRIVATE FUNCTION)

sub _parse_ietd_conf
{
    # return hashref
    my $hconf = {};
    
    my $keywords = join('|', @keywords);
    my $match = qr/($keywords)(.*)/;
    
    return (1, "config file doesn't exists") if (! -f $IETD_CONF_FILE );
    
    open my $fh, "<$IETD_CONF_FILE"
	or return (1, "unable to open config file : $!");
    
    my $currenttarget;
    
    while ( <$fh> )
    {
	next if m/^\s*#.*/;
	chomp;
	
	# new target ?
	my ( $newtarget ) = ( $_ =~ /\s*Target\s+(.*)/ );
	if ( $newtarget )
	{
	    $hconf->{$newtarget} = {};
	    $currenttarget = $newtarget;
	    next;
	}
	
	# target parameters
	my ($param, $value) = ( $_ =~ /$match/);
	if ( $param && $currenttarget)
	{
	    # cleanup value
 	    $value =~ s/^\s+//g;
 	    $value =~ s/\s+$//g;
	    
	    # case : Lun
	    if ( $param =~ /Lun/ )
	    {
		my ( $Lun, @lunparams) = split (/\s+|=|,/, $value);
		# convert to num
		$Lun += 0;
		my %lunparams = @lunparams;
		$hconf->{$currenttarget}{Luns}{$Lun} = \%lunparams;
	    }
	    # case : User
	    elsif ( $param =~ /(Outgoing|Incoming)User/ )
	    {
		my ( $user, $pass ) = split /\s+/, $value;
		$hconf->{$currenttarget}{$param} = { username => $user, password => $pass };
	    }
	    # default
	    else
	    {
		$hconf->{$currenttarget}{$param} = $value;
	    }
	}
	elsif ($param)
	{
	    if ( $param =~ /(Outgoing|Incoming)User/ )
	    {
		my ( $unused, $user, $pass ) = split /\s+/, $value;
		$hconf->{$GLOBAL_SECTION_NAME}->{$param} = { username => $user, password => $pass };
	    }
	    else
	    {
		$hconf->{$GLOBAL_SECTION_NAME}->{$param} = $value;
	    }
	}
    }
    
    return (0 ,$hconf);
}
    

sub _write_ietd_conf
{
    # this function should lock the config file
    my $hconf = shift;
    
    # save the comments (but won't keep them in place)
    open my $fh, "<$IETD_CONF_FILE"
	or return(1, "Unable to open config file : $!");

    my $outfile;

    while ( <$fh> ) { $outfile .= $_  if $_ =~ /^\#.*/ };
    close $fh;
    
    # I have to handle GLOBAL section first
    if (exists($hconf->{$GLOBAL_SECTION_NAME}))
    {
	my $tmp_hash = $hconf->{$GLOBAL_SECTION_NAME};
	foreach my $key (keys(%{$tmp_hash}))
	{
	    if ($key =~ /(Outgoing|Incoming)User/ )
	    {
		$outfile .= "$key $tmp_hash->{$key}->{'username'} $tmp_hash->{$key}->{'password'}\n";
	    }
	    else
	    {
		$outfile .= "$key $tmp_hash->{$key}\n";
	    }
	}
	delete $hconf->{$GLOBAL_SECTION_NAME};
    }
    

    # now write the configuration data
    while ( my ($target, $targetdata) = each %$hconf )
    {
	$outfile .= "\nTarget $target\n";
	# target Alias
	if ( exists $targetdata->{Alias} )
	{
	    $outfile .= "\tAlias $targetdata->{Alias}\n";
	    delete $targetdata->{Alias};
	}
	# Luns
	while ( my ($lun, $lundata ) = each %{$targetdata->{Luns}} ) {
	    $outfile .= "\tLun $lun ";
	    
	    if ( exists $lundata->{'Type'} and $lundata->{'Type'} ne "nullio" )
	    {
		$outfile .= "Type=$lundata->{Type}";
		$outfile .= ( -f $lundata->{'Path'} or -b $lundata->{'Path'} ) ? ",Path=$lundata->{Path}" : '' ;			
		$outfile .= exists $lundata->{'IOMode'} ? ",IOMode=$lundata->{IOMode}" : '' ;
                $outfile .= $lundata->{'ScsiId'} ? ",ScsiId=$lundata->{ScsiId}" : '' ;
                $outfile .= $lundata->{'ScsiSN'} ? ",ScsiSN=$lundata->{ScsiSN}" : '' ;
	    }
	    else
	    {
		$outfile .= "Type=nullio" ;
	    }
	    $outfile .= "\n";
	}
	
	delete $targetdata->{Luns};
	# now everything else
	while ( my ( $param, $value) = each %$targetdata )
	{
	    if ($param  =~ /(Outgoing|Incoming)User/ )
	    {
		$outfile .= "\t$param $value->{'username'} $value->{'password'}\n";
	    }
	    else
	    {
		$outfile .= "\t$param $value\n";
	    }
	}
    }
    
    open $fh, ">$IETD_CONF_FILE" or return(1, "Unable to open config file : $!");
    print $fh $outfile;
    close $fh;

    return (0, 0);
}


1;
