#!/usr/bin/perl

# $Id: edittarget.cgi 94 2009-02-19 16:29:16Z emmanuel $ 

use strict;
use warnings;

use Data::Dumper;

our %text;
our %config;
our %in;

require './iscsi-lib.pl';
&ReadParse();

# parse configuration files
my $conf=&parse_ietd_conf($config{'ietdconf'});


# ietd keywords
my %targetdefaults = (	
			HeaderDigest			=> 'None',
			DataDigest			=> 'None',
			MaxConnections			=> 1,
			InitialR2T			=> 'No',
			ImmediateData			=> 'Yes',
			MaxRecvDataSegmentLength	=> 65536,
			MaxXmitDataSegmentLength	=> 65536,
			MaxBurstLength			=> 262144,
			FirstBurstLength		=> 131072,
			DefaultTime2Wait		=> 10,
			DefaultTime2Retain		=> 20,
			MaxOutstandingR2T		=> 8,
			DataPDUInOrder			=> 'Yes',
			DataSequenceInOrder		=> 'Yes',
			ErrorRecoveryLevel		=> 0,
			Wthreads			=> 8,
			QueuedCommands			=> 64,
			IncomingUser			=> ,

) ;


##############################
# functions
sub addlun {
	# Display the form to add a new lun
	&header($text{'edittarget_addlun'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);

	# Display the lun details - used for add and edit.
	_print_lun('doaddlun');
}

sub doaddlun {
	# Apply the results from the add a new lun form
	
	# define new lun number
	my ( $max ) = sort { $b <=> $a } keys %{$conf->{$in{'target'}}{'Luns'}};
	$max = -1 unless $max ne '';
	$max++;
	
	# cancel if invalid device
	if ( $in{'luntype'} ne 'nullio' and not -f $in{'lundev'} and not -b $in{'lundev'}) {
		fail("invalid file :'$in{'lundev'}'");
	}

	if (not $in{'IOMode'}) {
		$in{'IOMode'}='wt'
	}

	# Set the field values 
	$conf->{$in{'target'}}{'Luns'}{$max}{'Path'} = $in{'lundev'};
	$conf->{$in{'target'}}{'Luns'}{$max}{'Type'} = $in{'luntype'};
        $conf->{$in{'target'}}{'Luns'}{$max}{'IOMode'} = $in{'IOMode'};
        $conf->{$in{'target'}}{'Luns'}{$max}{'ScsiId'} = $in{'lunscsiid'};
        $conf->{$in{'target'}}{'Luns'}{$max}{'ScsiSN'} = $in{'lunscsisn'};
	
	# Attempt to save the data to the ietd configuration file and display the outcome.
	eval { write_ietd_conf($config{'ietdconf'}, $conf) };
	if ($@) {
		fail($@);
	} else {
		&redirect("");
	}
}

sub dellun {
	#Display the form to delete an existing lun.
        my $nextaction = "dodellun";
        &header($text{'edittarget_dellun'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);

        print &ui_form_start("edittarget.cgi", "get");
        print &ui_table_start($text{'edittarget_dellun'}, undef, 2);

        print &ui_table_row($text{'edittarget_targetname'}, $in{'target'} );
        print &ui_table_row($text{'edittarget_lunid'}, $in{'lun'} );
        print &ui_table_row($text{'edittarget_delfile'}, &ui_checkbox("delfile",'selected', '', 'selected') );

	# Hidden fields to keep track of state.
        print &ui_hidden("action", "$nextaction");
        print &ui_hidden("target", $in{'target'});
        print &ui_hidden("lun", $in{'lun'});

        print &ui_table_end();
        print &ui_form_end([ [ "$nextaction", $text{'edittarget_apply'} ] ]);
}

sub dodellun {
	# Apply deletion
	
	# print "<pre>", Dumper \%in, "</pre>";
	
	# Delete the lun file if requested to do so.
	if ( $in{'delfile'} and ( -f $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Path'} )) {
		print "<h3>Deleting $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Path'} </h3>";
		# delete the file if it's not a device
		unlink $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Path'} 
			or print "<h3>$text{'edittarget_delfileerror'} : $!</h3>";
	}
	
	delete $conf->{$in{'target'}}{'Luns'}{$in{'lun'}};

        # Attempt to save the data to the ietd configuration file and display the outcome.	
	eval { write_ietd_conf($config{'ietdconf'}, $conf) };
	if ($@) {
		fail($@);
	} else {
		redirect("");
	}
}


sub editlun {
	# Display the 'Edit Lun' form.
	&header($text{'edittarget_editlun'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),undef,undef,$text{'edittarget_lunid'}." $in{'lun'}");

        # Display the lun details - used for add and edit.
	_print_lun('doeditlun');
}

sub doeditlun {
	# apply edits
	
	$conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Path'} = $in{'lundev'};
	$conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Type'} = $in{'luntype'};
	$conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'IOMode'} = $in{'IOMode'};
        $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'ScsiId'} = $in{'lunscsiid'};
        $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'ScsiSN'} = $in{'lunscsisn'};
	
        # Attempt to save the data to the ietd configuration file and display the outcome.
	eval { write_ietd_conf($config{'ietdconf'}, $conf) };
	if ($@) {
		fail($@);
	} else {
		&redirect("");
	}
}

sub addtarget {
	# Display the 'Add Target' form
	&header($text{'edittarget_addtarget'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);

        # Display the target details - used for add and edit.
	_print_target('doaddtarget');
}

sub doaddtarget {
	
	# Apply the results returned from the 'Add Target' form

	# Validate the target name before adding.	
	if ( $in{targetname} eq "" ) {	
		fail($text{'edittarget_missingtargetname'});
	} elsif ( exists $conf->{$in{targetname}} ) {
		fail($text{'edittarget_addexist'});
	} elsif  ( $in{'targetname'} =~ /(\w|\.|\+|-)+/ ) {
		$conf->{$in{targetname}} = {};
		if ( defined $in{targetalias} ) {
			$conf->{$in{targetname}}{Alias} = $in{targetalias} ;
		}
		if ($in{IncomingUser_password}) {
			my $passlength = length($in{IncomingUser_password});
			if ( $passlength < 12 || $passlength > 16 ) {
				fail($text{'edittarget_errorpass'});
			}
		}
		$conf->{$in{targetname}}{HeaderDigest} = $in{HeaderDigest}  ;
		$conf->{$in{targetname}}{DataDigest} = $in{DataDigest} ;
		$conf->{$in{targetname}}{MaxConnections} = $in{MaxConnections} ;
		$conf->{$in{targetname}}{InitialR2T} = $in{InitialR2T} ;
		$conf->{$in{targetname}}{ImmediateData} = $in{ImmediateData} ;
		$conf->{$in{targetname}}{MaxRecvDataSegmentLength} = $in{MaxRecvDataSegmentLength} ;
		$conf->{$in{targetname}}{MaxXmitDataSegmentLength} = $in{MaxXmitDataSegmentLength} ;
		$conf->{$in{targetname}}{MaxBurstLength} = $in{MaxBurstLength} ;
		$conf->{$in{targetname}}{FirstBurstLength} = $in{FirstBurstLength} ;
		$conf->{$in{targetname}}{DefaultTime2Wait} = $in{DefaultTime2Wait} ;
		$conf->{$in{targetname}}{DefaultTime2Retain} = $in{DefaultTime2Retain} ;
		$conf->{$in{targetname}}{MaxOutstandingR2T} = $in{MaxOutstandingR2T} ;
		$conf->{$in{targetname}}{DataPDUInOrder} = $in{DataPDUInOrder} ;
		$conf->{$in{targetname}}{DataSequenceInOrder} = $in{DataSequenceInOrder} ;
		$conf->{$in{targetname}}{ErrorRecoveryLevel} = $in{ErrorRecoveryLevel} ;
		$conf->{$in{targetname}}{Wthreads} = $in{Wthreads} ;
		$conf->{$in{targetname}}{QueuedCommands} = $in{QueuedCommands} ;
		$conf->{$in{targetname}}{IncomingUser}{username} = $in{IncomingUser_username} ;
		$conf->{$in{targetname}}{IncomingUser}{password} = $in{IncomingUser_password} ;

                # Attempt to save the data to the ietd configuration file and display the outcome.
		eval { write_ietd_conf($config{'ietdconf'}, $conf) };
		if ($@) {
			fail($@);
		} else {
			&redirect("");
		}
	} else {
		fail($text{'edittarget_invalidtgtname'});
	}
}


sub deltarget {
	# Display the 'Delete Target' form
	my $nextaction = "dodeltarget";
	&header($text{'edittarget_deltarget'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);

	# Check that there are no luns defined against the target before delete.
        if ( defined $conf->{$in{target}}{'Luns'} ) {
                print '<br><hr><br><h3>' . $text{'edittarget_notempty'} . '</h3>';
                print "\n<br><a href=index.cgi>$text{auth_index}</a><br><BR>\n";
        } else {

                print &ui_form_start("edittarget.cgi", "get");
                print &ui_table_start($text{'edittarget_deltarget'}, undef, 2);

                print &ui_table_row($text{'edittarget_targetname'}, $in{'target'} );

                print &ui_hidden("action", "$nextaction");
                print &ui_hidden("target", $in{'target'});
                print &ui_hidden("lun", $in{'lun'});

                print &ui_table_end();
                print &ui_form_end([ [ "$nextaction", $text{'edittarget_apply'} ] ]);

               print "<br><br><hr>\n<a href=index.cgi>$text{auth_index}</a><br><BR>\n";
	}
}

sub dodeltarget {
	# Apply deletion
		
	# Remove the target from in-memory cache
	delete $conf->{$in{'target'}};
	
        # Attempt to save the data to the ietd configuration file and display the outcome.
	eval { write_ietd_conf($config{'ietdconf'}, $conf) };
	if ($@) {
		fail($@);
	} else {
		&redirect("");
	}
}

sub edittarget {
	# Display the 'Edit Target' form
	&header($text{'edittarget_title'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);

        # Display the target details - used for add and edit.
	_print_target('doedittarget');	
}

sub doedittarget {
	# Apply edits
	
	# print "<pre>",Dumper $conf, "</pre>";
	
	
	$conf->{$in{'target'}}{'Alias'} = $in{'targetalias'}
		if  $in{'targetalias'} =~ /(\w|\.|\+|-)+/ ;
	
	if ( $in{'targetname'} =~ /(\w|\.|\+|-)+/ 
		and $in{'target'} ne $in{'targetname'} ) {
		$conf->{$in{'targetname'}} = $conf->{$in{'target'}} ;
		delete $conf->{$in{'target'}};
        	$conf->{$in{'targetname'}}{'HeaderDigest'} = $in{'HeaderDigest'};
        	$conf->{$in{'targetname'}}{'DataDigest'} = $in{'DataDigest'};
        	$conf->{$in{'targetname'}}{'MaxConnections'} = $in{'MaxConnections'};
        	$conf->{$in{'targetname'}}{'InitialR2T'} = $in{'InitialR2T'};
        	$conf->{$in{'targetname'}}{'ImmediateData'} = $in{'ImmediateData'};
        	$conf->{$in{'targetname'}}{'MaxRecvDataSegmentLength'} = $in{'MaxRecvDataSegmentLength'};
        	$conf->{$in{'targetname'}}{'MaxXmitDataSegmentLength'} = $in{'MaxXmitDataSegmentLength'};
        	$conf->{$in{'targetname'}}{'MaxBurstLength'} = $in{'MaxBurstLength'};
        	$conf->{$in{'targetname'}}{'FirstBurstLength'} = $in{'FirstBurstLength'};
        	$conf->{$in{'targetname'}}{'DefaultTime2Wait'} = $in{'DefaultTime2Wait'};
        	$conf->{$in{'targetname'}}{'DefaultTime2Retain'} = $in{'DefaultTime2Retain'};
        	$conf->{$in{'targetname'}}{'MaxOutstandingR2T'} = $in{'MaxOutstandingR2T'};
        	$conf->{$in{'targetname'}}{'DataPDUInOrder'} = $in{'DataPDUInOrder'};
        	$conf->{$in{'targetname'}}{'DataSequenceInOrder'} = $in{'DataSequenceInOrder'};
        	$conf->{$in{'targetname'}}{'ErrorRecoveryLevel'} = $in{'ErrorRecoveryLevel'};
        	$conf->{$in{'targetname'}}{'Wthreads'} = $in{'Wthreads'};
        	$conf->{$in{'targetname'}}{'QueuedCommands'} = $in{'QueuedCommands'};
        	#$conf->{$in{'targetname'}}{'IncomingUser'} = $in{'IncomingUser'};
        	$conf->{$in{'targetname'}}{'IncomingUser'}{'username'} = $in{'IncomingUser_username'};
        	$conf->{$in{'targetname'}}{'IncomingUser'}{'password'} = $in{'IncomingUser_password'};
	}
	else {
        	$conf->{$in{'target'}}{'HeaderDigest'} = $in{'HeaderDigest'};
        	$conf->{$in{'target'}}{'DataDigest'} = $in{'DataDigest'};
        	$conf->{$in{'target'}}{'MaxConnections'} = $in{'MaxConnections'};
        	$conf->{$in{'target'}}{'InitialR2T'} = $in{'InitialR2T'};
        	$conf->{$in{'target'}}{'ImmediateData'} = $in{'ImmediateData'};
        	$conf->{$in{'target'}}{'MaxRecvDataSegmentLength'} = $in{'MaxRecvDataSegmentLength'};
        	$conf->{$in{'target'}}{'MaxXmitDataSegmentLength'} = $in{'MaxXmitDataSegmentLength'};
        	$conf->{$in{'target'}}{'MaxBurstLength'} = $in{'MaxBurstLength'};
        	$conf->{$in{'target'}}{'FirstBurstLength'} = $in{'FirstBurstLength'};
        	$conf->{$in{'target'}}{'DefaultTime2Wait'} = $in{'DefaultTime2Wait'};
        	$conf->{$in{'target'}}{'DefaultTime2Retain'} = $in{'DefaultTime2Retain'};
        	$conf->{$in{'target'}}{'MaxOutstandingR2T'} = $in{'MaxOutstandingR2T'};
        	$conf->{$in{'target'}}{'DataPDUInOrder'} = $in{'DataPDUInOrder'};
        	$conf->{$in{'target'}}{'DataSequenceInOrder'} = $in{'DataSequenceInOrder'};
        	$conf->{$in{'target'}}{'ErrorRecoveryLevel'} = $in{'ErrorRecoveryLevel'};
        	$conf->{$in{'target'}}{'Wthreads'} = $in{'Wthreads'};
        	$conf->{$in{'target'}}{'QueuedCommands'} = $in{'QueuedCommands'};
        	#$conf->{$in{'target'}}{'IncomingUser'} = $in{'IncomingUser'};
        	$conf->{$in{'target'}}{'IncomingUser'}{'username'} = $in{'IncomingUser_username'};
        	$conf->{$in{'target'}}{'IncomingUser'}{'password'} = $in{'IncomingUser_password'};
	}		
	# print "<pre>",Dumper $conf, "</pre>";
	
        # Attempt to save the data to the ietd configuration file and display the outcome.
	eval { write_ietd_conf($config{'ietdconf'}, $conf) };
	if ($@) {
		fail($@)
	} else {
		&redirect("");
	}
}


########################
sub _print_lun {

	my $nextaction=shift;

	# get LUN info from configuration OR CGI params
	my $lunfile = $in{'lundev'} || $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Path'};
	my $luntype = $in{'luntype'} || $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Type'};
	my $luniomode = $in{'IOMode'} || $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'IOMode'};
	my $lunscsiid = $in{'lunscsiid'} || $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'ScsiId'};
    my $lunscsisn = $in{'lunscsisn'} || $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'ScsiSN'};
	
	my $createlun = "<a href=\"createlun.cgi?target=$in{'target'}\">$text{'edittarget_createdev'}</a>";
	
	if ( $in{action} eq 'editlun' ) {
	$createlun = "<a href=\"createlun.cgi\?action=resizedev&target=$in{'target'}&lundev=$lunfile&lun=$in{lun}\">$text{'edittarget_resizedev'}</a>"
	}
	
	print qq(<script src="tools.js"></script>);

	if ( $luntype !~ /fileio/ ) {
		print qq(
        <script language="JavaScript" type="text/javascript">
        window.onload=function () {
        	var elements = document.getElementsByName('luntype');
        	elements[0].onchange = setLunParams;
        	setLunParams();
        };
        </script>);
	}
	
	print &ui_form_start("edittarget.cgi", "get");
	print &ui_table_start($text{'lun_header'}, undef, 2);

	print &ui_table_row($text{'edittarget_luntype'},
	        &ui_select("luntype", $luntype, 
			[ ["nullio", $text{'edittarget_luntypenull'} ],
			  ["blockio", $text{'edittarget_luntypeblock'} ],
			  ["fileio", $text{'edittarget_luntypefile'} ] ]));

        print &ui_table_start("", "id='lunparams' width='100%'", 2);

	print &ui_table_row($text{'edittarget_lundev'}, &ui_textbox("lundev", $lunfile, 40)."&nbsp;".&file_chooser_button("lundev") . $createlun );
	
		
	print &ui_table_row($text{'edittarget_lunscsiid'}, &ui_textbox("lunscsiid", $lunscsiid, 40));
	print &ui_table_row($text{'edittarget_lunscsisn'}, &ui_textbox("lunscsisn", $lunscsisn, 40));
	print &ui_table_row($text{'edittarget_iomode'},
        	&ui_radio("IOMode", $luniomode, 
			[ ["wb", $text{'edittarget_iomodewb'} ."<br>" ],
			  ["wt", $text{'edittarget_iomodewt'}."<br>" ],
			  ["ro", $text{'edittarget_iomodero'} ."<br>" ] ]));

	print &ui_hidden("action", $nextaction);
	print &ui_hidden("target", $in{'target'});
	print &ui_hidden("lun", $in{'lun'});
        print &ui_table_end();

	print &ui_table_end();
        print &ui_form_end([ [ "$nextaction", $nextaction eq "doaddlun" ? $text{'edittarget_create'} : $text{'edittarget_apply'} ] ]);

        print "<hr>\n<a href=index.cgi>$text{auth_index}</a><br><br>\n";
}


sub _print_target {
	my $nextaction=shift;

        my $host = "";
        my @hostarray = split(/\./, `hostname -f`);
		chomp @hostarray;

        for (my $i = @hostarray; $i > 0; $i--) { $host = $host.'.'.$hostarray[$i-1]; }

	my $month = `date +%m`;
	chomp $month;
	my $year = `date +%Y`;
	chomp $year;
	my $targetname=$in{'target'} || "iqn.$year-$month.com.yourdomain:target.name" ;
	my @temp =  split ( /.*:/, $targetname );
	my $targetalias=$conf->{$in{'target'}}{'Alias'}|| $temp[$#temp];

	my $headerdigest=$in{'HeaderDigest'} || $conf->{$in{'target'}}{'HeaderDigest'};
	if (!$headerdigest) {
		$headerdigest=$targetdefaults{'HeaderDigest'};
	}
	my $datadigest=$in{'DataDigest'} || $conf->{$in{'target'}}{'DataDigest'};
	if (!$datadigest) {
		$datadigest=$targetdefaults{'DataDigest'};
	}
	my $maxconnections=$in{'MaxConnections'} || $conf->{$in{'target'}}{'MaxConnections'};
	if (!$maxconnections) {
		$maxconnections=$targetdefaults{'MaxConnections'};
	}
	my $initialr2t=$in{'InitialR2T'} || $conf->{$in{'target'}}{'InitialR2T'};
	if (!$initialr2t) {
		$initialr2t=$targetdefaults{'InitialR2T'};
	}
	my $immediatedata=$in{'ImmediateData'} || $conf->{$in{'target'}}{'ImmediateData'};
	if (!$immediatedata) {
		$immediatedata=$targetdefaults{'ImmediateData'};
	}
	my $maxrecvdatasegmentlength=$in{'MaxRecvDataSegmentLength'} || $conf->{$in{'target'}}{'MaxRecvDataSegmentLength'};
	if (!$maxrecvdatasegmentlength) {
		$maxrecvdatasegmentlength=$targetdefaults{'MaxRecvDataSegmentLength'};
	}
	my $maxxmitdatasegmentlength=$in{'MaxXmitDataSegmentLength'} || $conf->{$in{'target'}}{'MaxXmitDataSegmentLength'};
	if (!$maxxmitdatasegmentlength) {
		$maxxmitdatasegmentlength=$targetdefaults{'MaxXmitDataSegmentLength'};
	}
	my $maxburstlength=$in{'MaxBurstLength'} || $conf->{$in{'target'}}{'MaxBurstLength'};
	if (!$maxburstlength) {
		$maxburstlength=$targetdefaults{'MaxBurstLength'};
	}
	my $firstburstlength=$in{'FirstBurstLength'} || $conf->{$in{'target'}}{'FirstBurstLength'};
	if (!$firstburstlength) {
		$firstburstlength=$targetdefaults{'FirstBurstLength'};
	}
	my $defaulttime2wait=$in{'DefaultTime2Wait'} || $conf->{$in{'target'}}{'DefaultTime2Wait'};
	if (!$defaulttime2wait) {
		$defaulttime2wait=$targetdefaults{'DefaultTime2Wait'};
	}
	my $defaulttime2retain=$in{'DefaultTime2Retain'} || $conf->{$in{'target'}}{'DefaultTime2Retain'};
	if (!$defaulttime2retain) {
		$defaulttime2retain=$targetdefaults{'DefaultTime2Retain'};
	}
	my $maxoutstandingr2t=$in{'MaxOutstandingR2T'} || $conf->{$in{'target'}}{'MaxOutstandingR2T'};
	if (!$maxoutstandingr2t) {
		$maxoutstandingr2t=$targetdefaults{'MaxOutstandingR2T'};
	}
	my $datapduinorder=$in{'DataPDUInOrder'} || $conf->{$in{'target'}}{'DataPDUInOrder'};
	if (!$datapduinorder) {
		$datapduinorder=$targetdefaults{'DataPDUInOrder'};
	}
	my $datasequenceinorder=$in{'DataSequenceInOrder'} || $conf->{$in{'target'}}{'DataSequenceInOrder'};
	if (!$datasequenceinorder) {
		$datasequenceinorder=$targetdefaults{'DataSequenceInOrder'};
	}
	my $errorrecoverylevel=$in{'ErrorRecoveryLevel'} || $conf->{$in{'target'}}{'ErrorRecoveryLevel'};
	if (!$errorrecoverylevel) {
		$errorrecoverylevel=$targetdefaults{'ErrorRecoveryLevel'};
	}
	my $wthreads=$in{'Wthreads'} || $conf->{$in{'target'}}{'Wthreads'};
	if (!$wthreads) {
		$wthreads=$targetdefaults{'Wthreads'};
	}
	my $queuedcommands=$in{'QueuedCommands'} || $conf->{$in{'target'}}{'QueuedCommands'};
	if (!$queuedcommands) {
		$queuedcommands=$targetdefaults{'QueuedCommands'};
	}

	my $incominguserusername=$in{'IncomingUser_username'} || $conf->{$in{'target'}}{'IncomingUser'}{'username'}; 
	my $incominguserpassword=$in{'IncomingUser_password'} || $conf->{$in{'target'}}{'IncomingUser'}{'password'}; 

        print &ui_form_start("edittarget.cgi", "get");
        print &ui_table_start($text{'edittarget_targetinfo'}, undef, 2);

        print &ui_table_row($text{'edittarget_targetname'}, &ui_textbox("targetname", $targetname, 50));
        print &ui_table_row($text{'edittarget_targetalias'}, &ui_textbox("targetalias", $targetalias, 50));
        
	print &ui_table_row($text{'edittarget_headerdigest'},
                &ui_select("HeaderDigest", $headerdigest,
                        [ ["None", "None<br>"], ["CRC32", "CRC32<br>"] ]));
	print &ui_table_row($text{'edittarget_datadigest'},
                &ui_select("DataDigest", $datadigest,
                        [ ["None", "None<br>"], ["CRC32", "CRC32<br>"] ]));
	print &ui_table_row($text{'edittarget_maxconnections'},
                &ui_select("MaxConnections", $maxconnections,
                        [ ["1", "1<br>"] ]).$text{'edittarget_maxconnectionsdesc'} );
	print &ui_table_row($text{'edittarget_initialr2t'},
                &ui_select("InitialR2T", $initialr2t,
                        [ ["Yes", "Yes<br>"], ["No", "No<br>"] ]));
	print &ui_table_row($text{'edittarget_immediatedata'},
                &ui_select("ImmediateData", $immediatedata,
                        [ ["Yes", "Yes<br>"], ["No", "No<br>"] ]));
        print &ui_table_row($text{'edittarget_maxrecvdatasegmentlength'},
                &ui_select("MaxRecvDataSegmentLength", $maxrecvdatasegmentlength,
                       [ ["2048", "2048<br>"], ["4096", "4096<br>"], ["8192", "8192<br>"], ["16384", "16384<br>"],
                         ["32768", "32768<br>"], ["65536", "65536<br>"], ["131072", "131072<br>"], ["262144", "262144<br>"] ]));
        print &ui_table_row($text{'edittarget_maxxmitdatasegmentlength'},
                &ui_select("MaxXmitDataSegmentLength", $maxxmitdatasegmentlength,
                       [ ["2048", "2048<br>"], ["4096", "4096<br>"], ["8192", "8192<br>"], ["16384", "16384<br>"],
                         ["32768", "32768<br>"], ["65536", "65536<br>"], ["131072", "131072<br>"], ["262144", "262144<br>"] ]));
        print &ui_table_row($text{'edittarget_maxburstlength'},
                &ui_select("MaxBurstLength", $maxburstlength,
                       [ ["2048", "2048<br>"], ["4096", "4096<br>"], ["8192", "8192<br>"], ["16384", "16384<br>"],
                         ["32768", "32768<br>"], ["65536", "65536<br>"], ["131072", "131072<br>"], ["262144", "262144<br>"] ]));
        print &ui_table_row($text{'edittarget_firstburstlength'},
                &ui_select("FirstBurstLength", $firstburstlength,
                       [ ["2048", "2048<br>"], ["4096", "4096<br>"], ["8192", "8192<br>"], ["16384", "16384<br>"],
                         ["32768", "32768<br>"], ["65536", "65536<br>"], ["131072", "131072<br>"], ["262144", "262144<br>"] ]));
        print &ui_table_row($text{'edittarget_defaulttime2wait'},
                &ui_select("DefaultTime2Wait", $defaulttime2wait,
                        [ ["1", "1<br>"], ["2", "2<br>"], ["3", "3<br>"], ["4", "4<br>"], ["5", "5<br>"],
                          ["6", "6<br>"], ["7", "7<br>"], ["8", "8<br>"], ["9", "9<br>"], ["10", "10<br>"] ]));
        print &ui_table_row($text{'edittarget_defaulttime2retain'},
                &ui_select("DefaultTime2Retain", $defaulttime2retain,
                        [ ["1", "1<br>"], ["2", "2<br>"], ["3", "3<br>"], ["4", "4<br>"], ["5", "5<br>"],
                          ["6", "6<br>"], ["7", "7<br>"], ["8", "8<br>"], ["9", "9<br>"], ["10", "10<br>"],
                          ["11", "11<br>"], ["12", "12<br>"], ["13", "13<br>"], ["14", "14<br>"], ["15", "15<br>"],
                          ["16", "16<br>"], ["17", "17<br>"], ["18", "18<br>"], ["19", "19<br>"], ["20", "20<br>"] ]));
        print &ui_table_row($text{'edittarget_maxoutstandingr2t'},
                &ui_select("MaxOutstandingR2T", $maxoutstandingr2t,
                        [ ["1", "1<br>"], ["2", "2<br>"], ["3", "3<br>"], ["4", "4<br>"], ["5", "5<br>"],
                          ["6", "6<br>"], ["7", "7<br>"], ["8", "8<br>"], ["9", "9<br>"], ["10", "10<br>"] ]));
	print &ui_table_row($text{'edittarget_datapduinorder'},
                &ui_select("DataPDUInOrder", $datapduinorder,
                        [ ["Yes", "Yes<br>"], ["No", "No<br>"] ]));
	print &ui_table_row($text{'edittarget_datasequenceinorder'},
                &ui_select("DataSequenceInOrder", $datasequenceinorder,
                        [ ["Yes", "Yes<br>"], ["No", "No<br>"] ]));
	print &ui_table_row($text{'edittarget_errorrecoverylevel'},
                &ui_select("ErrorRecoveryLevel", $errorrecoverylevel,
                        [ ["0", "0<br>"] ]).$text{'edittarget_errorrecoveryleveldesc'} );
        print &ui_table_row($text{'edittarget_wthreads'},
                &ui_select("Wthreads", $wthreads,
                        [ ["1", "1<br>"], ["2", "2<br>"], ["3", "3<br>"], ["4", "4<br>"],
                          ["5", "5<br>"], ["6", "6<br>"], ["7", "7<br>"], ["8", "8<br>"] ]).$text{'edittarget_wthreadsdesc'} );
        print &ui_table_row($text{'edittarget_queuedcommands'},
                &ui_select("QueuedCommands", $queuedcommands,
                        [ ["1", "1<br>"], ["2", "2<br>"], ["4", "4<br>"], ["8", "8<br>"],
                          ["16", "16<br>"], ["32", "32<br>"], ["64", "64<br>"] ]));
        print &ui_table_row($text{'edittarget_incominguserusername'}, &ui_textbox("IncomingUser_username", $incominguserusername, 50));
        print &ui_table_row('', $text{'edittarget_incominguserdesc3'});
        print &ui_table_row($text{'edittarget_incominguserpassword'}, &ui_password("IncomingUser_password", $incominguserpassword, 50));
        print &ui_table_row('', $text{'edittarget_incominguserdesc2'});

        print &ui_hidden("action", $nextaction);
        print &ui_hidden("target", $in{'target'});

        print &ui_table_end();
        print &ui_form_end([ [ "$nextaction", $nextaction eq "doaddtarget" ? $text{'edittarget_create'} : $text{'edittarget_apply'} ] ]);

        print "<hr>\n<a href=index.cgi>$text{auth_index}</a><br><br>\n";
}

##############################
# failure!

sub fail {
	&header(undef, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);
	my $message=shift;
	print '<h3>' . $text{'edittarget_failed'} . '</h3><br>';
	print '<h3>' . $message . '</h3>';
	print "<hr>\n<a href=index.cgi>$text{auth_index}</a><br><br>\n";
	# footer
	&footer("/", $text{'index'});
	exit;
}

##############################

# actions
my %actions = (
	addlun 		=> \&addlun,
	doaddlun 	=> \&doaddlun,
	dellun 		=> \&dellun,
	dodellun 	=> \&dodellun,
	editlun		=> \&editlun,
	doeditlun	=> \&doeditlun,
	addtarget 	=> \&addtarget,
	doaddtarget	=> \&doaddtarget,
	deltarget 	=> \&deltarget,
	dodeltarget => \&dodeltarget,
	edittarget	=> \&edittarget,
	doedittarget	=> \&doedittarget,
) ;

##############################
if ( exists $actions{$in{action}} ) {
	$actions{$in{action}}->();
} else {
	edittarget();
}

# test
# print "<pre>";
# print Dumper \%in;
# print "</pre>";

# footer
&footer("/", $text{'index'});


# end
exit;
