#!/usr/bin/perl
# Display and configure iSCSI target volumes 
#****h*
# NAME ietadm
# iSCSI Enterprise Target Webmin Module
# 
# FUNCTION
# manage your IET service and configuration easily.
# 
# HISTORY
# * V 0.9.5 on 11/05/03		added CHAP authentication with password length check, fixed fail sub for some errors, fixed wthru -> wt
#				made targetname and lun clickable for fast edit, moved ReadParse(); out of iscsi-lib.pl
#				added the ability to edit the targets.allow and initiators.allow files manually
# * V 0.9.4 on 10/04/14		added version info, targetparameters, removed 'Changes applied' screen show main page instead,
#                               made use of targetdefaults, changed default targetname, changed default path to /etc/iet/*
# * V 0.9.3 on 09/05/29		bugs corrections (Luke Suchocki), configuration backup (Caspar Smit)
# * V 0.9.2 on 09/02/19		bugs corrections, error checking
# * V 0.9.1 on 08/07/03		bugs corrections
# * V 0.9.0 on 08/06/23		bugs corrections
# * V 0.8.0 on 08/06/17		LUN file resizing
# * V 0.7.0 on 08/06/16		LUN file deletion
# * V 0.6.0 on 08/03/20		LUN file creation
# * V 0.5.0 on 08/03/19		added theming, SCSIID and SCSISN (A. Wood)
# * V 0.4.0 on 06/06/06		session management
# * V 0.3.0 on 06/05/29		add and delete LUNs
# * V 0.2.0 on 06/05/22		edit existing LUNs
# * V 0.1.0 on 06/05/12		initial release
# 
# BUGS
# None known. Comments welcome :)
# 
# TODO
# * display targets status (active/inactive)
# * activate/deactivate targets
# * initiators.allow/deny management
# * apply/dump settings live (using ietadm)
# 
# LICENSE
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   the LICENSE file for details.
#
# REVISION
# $Id: index.cgi 97 2009-05-29 13:12:07Z emmanuel $
#
# AUTHOR
# see AUTHORS file
#
# COPYRIGHT
# (c) 2009 Intellique (http://www.intellique.com)
#***


use strict;
use warnings;

use Data::Dumper;
our %text;
our %config;
our %in;

require './iscsi-lib.pl';
&ReadParse();

&ui_print_header(undef, $text{'index_title'}, "", undef, 1, 1, 0,
        &help_search_link("ietd", "man", "doc"),
        undef, undef, &get_ietd_version() );

#links to add a target | session management | auth configuration
print "| <a href=\"edittarget.cgi?action=addtarget\">$text{'index_add'}</a> | <a href=sessions.cgi>$text{index_session}</a> | <a href=edit_targetsallow.cgi>$text{index_targetsallow}</a> | <a href=edit_initiatorsallow.cgi>$text{index_initiatorsallow}</a> |";
# print "| <a href=authconfig.cgi>$text{index_auth}</a>"; # work in progres...
print "<br>\n";

# parse configuration files
if ( not -f $config{'ietdconf'} ) {
	print "<h3>" . $text{'index_noconf'} . "</h3>\n";
	&footer("/", $text{'index'});
	exit;
}

my $conf=&parse_ietd_conf($config{'ietdconf'});

if ( not $conf ) {
	print "<h3>" . $text{'index_none'} . "</h3>\n";
} else {
	# display table header
	print ui_columns_start([ $text{'index_targetname'},$text{'index_alias'},$text{'index_targetlun'},$text{'index_luntype'},$text{'index_lundev'},"&nbsp;" ], 90);
	
	my $editLink="";
	my $deleteLink="";
	foreach my $target ( sort keys %$conf ) {
		# display target
		$editLink = "<a href='edittarget.cgi?action=edittarget&amp;target=$target'>$text{'index_edittarget'}</a>";
		$deleteLink = "<a href=\"edittarget.cgi?target=$target&amp;action=deltarget\">$text{'index_deltarget'}</a>";
		print ui_columns_row([ "<a href='edittarget.cgi?action=edittarget&amp;target=$target'>$target</a>", $conf->{$target}{'Alias'}, "&nbsp;", "$editLink &nbsp; $deleteLink" ], [ "", "", "colspan=3", "" ]);

		# display LUNs
		foreach my $lun ( sort { $a <=> $b } keys %{$conf->{$target}{'Luns'}} ) {

			$editLink = "<a href=\"edittarget.cgi?action=editlun&amp;target=$target&amp;lun=$lun\">$text{'index_edittarget'}</a>";
			$deleteLink = "<a href=\"edittarget.cgi?target=$target&amp;lun=$lun&amp;action=dellun\">$text{'index_deltarget'}</a>";
			print ui_columns_row([ "&nbsp;", "<a href=\"edittarget.cgi?action=editlun&amp;target=$target&amp;lun=$lun\">$lun</a>", $conf->{$target}{'Luns'}{$lun}{'Type'}, $conf->{$target}{'Luns'}{$lun}{'Path'}, "$editLink &nbsp; $deleteLink" ],
				[ "colspan=2", "", "", "", ""]);
		}

		# add a LUN entry and targets separation
		$editLink = "<a href=\"edittarget.cgi?target=$target&amp;action=addlun\">$text{'index_addlun'}</a>";
		print ui_columns_row([ "&nbsp;", "$editLink" ], [ "colspan=5", "" ]);
	}
	
	print ui_columns_end();
}

# status, start, stop and restart service
if ( $in{'start'} ) {
	ietd_start()	
} elsif ( $in{'stop'} ) {
	ietd_stop()	
} elsif ( $in{'restart'} ) {
	ietd_restart()	
}

my $status = ietd_status() ;
my @statusmess = ( $text{'index_notrunning'}, $text{'index_running'} );
my @startstop = ( "start" , "stop" );
my @startstopmess = ( $text{'index_start'}, $text{'index_stop'} );

print <<EOF;
<form action="index.cgi" name="startstop" method="get">
<table border="0" align="center">
  <tr><td colspan='2' align="center"><b>$text{'index_status'}:</b> $statusmess[$status]</td></tr>
  <tr>
	<td><input type="submit" name="$startstop[$status]" value="$startstopmess[$status]"></td>
EOF

if ( $status ) { print "<td><input type=\"submit\" name=\"restart\" value=\"$text{'index_restart'}\"></td>"; }

print <<EOF;
  </tr>
</table>
</form>
EOF

# end
&footer("/", $text{'index'});
