#!/usr/bin/perl

do '../web-lib.pl';
do '../ui-lib.pl';

use strict;
use warnings;

our %text;
our %config;


# ietd keywords
my @keywords=qw( Lun Alias IncomingUser OutgoingUser MaxConnections InitialR2T ImmediateData MaxRecvDataSegmentLength  MaxXmitDataSegmentLength MaxBurstLength FirstBurstLength DefaultTime2Wait DefaultTime2Retain MaxOutstandingR2T DataPDUInOrder DataSequenceInOrder ErrorRecoveryLevel HeaderDigest DataDigest Wthreads QueuedCommands );	


# subs
sub parse_ietd_conf {
	# This sub IGNORES default User settings (discovery user)
	my $conf=shift;
	
	# return hashref
	my $hconf;
	
	my $keywords=join('|', @keywords);
	my $match=qr/($keywords)(.*)/;
	
	if ( -f $conf ) {
		open my $fh, "<$conf";
		my $currenttarget;
		
		
		while ( <$fh> ) {
		     next if m/^\s*#.*/;
			 chomp;
			 
			 # new target ?
			 my ( $newtarget ) = ( $_ =~ /\s*Target\s+(.*)/ );
			 if ( $newtarget ) {
			 	$hconf->{$newtarget} = {};
				$currenttarget = $newtarget;
				next
			}
			
			# target parameters
			my ($param, $value) = ( $_ =~ /$match/);
			if ( $param ) {
				# cleanup value
				$value =~ s/(^\s+)|(\s+$)//g;
				# case : Lun
				if ( $param =~ /Lun/ ) {
					my ( $Lun, @lunparams) = split (/\s+|=|,/, $value);
					# convert to num
					$Lun += 0;
					my %lunparams = @lunparams;
					$hconf->{$currenttarget}{Luns}{$Lun} = \%lunparams;
					
				}
				# case : User
				elsif ( $param =~ /(Outgoing|Incoming)User/ ) {
					my ( $user, $pass ) = split /\s+/, $value;
					$hconf->{$currenttarget}{$param} = { username => $user, password => $pass };
				}
				# default
				else {
					$hconf->{$currenttarget}{$param} = $value;
				}
			}
			
			
			 
		}
		
	
	} else {
		return
	}

	return $hconf;
}


sub write_ietd_conf {
	# this function should lock the config file
	my $conffile = shift;
	my $hconf = shift;
	
	# save the comments (but won't keep them in place)
	open my $fh, "<$conffile" or die "Unable to open file" . ':<br />' . $!;
	my $outfile;
	while ( <$fh> ) { $outfile .= $_  if $_ =~ /^#.*/ };
	close $fh;
	
	# print "<pre>", Dumper $hconf, "</pre>";
	
	# now write the configuration data
	while ( my ($target, $targetdata) = each %$hconf ) {
		print  "\n ietadm --op new --tid=$hconf --params Name=$target \n";

		$outfile .= "\nTarget $target\n";
		# target Alias
		if ( exists $targetdata->{Alias} ) {
			$outfile .= "\tAlias $targetdata->{Alias}\n";
			delete $targetdata->{Alias};
		}
		# Luns
		while ( my ($lun, $lundata ) = each %{$targetdata->{Luns}} ) {
			$outfile .= "\tLun $lun ";
			
			if ( exists $lundata->{'Type'} and $lundata->{'Type'} ne "nullio" ) {
				$outfile .= "Type=$lundata->{Type}";
				$outfile .= ( -f $lundata->{'Path'} or -b $lundata->{'Path'} ) ? ",Path=$lundata->{Path}" : '' ;			
				$outfile .= exists $lundata->{'IOMode'} ? ",IOMode=$lundata->{IOMode}" : '' ;
			} else {
				$outfile .= "Type=nullio" ;
			}
			$outfile .= "\n";
		}
		
		delete $targetdata->{Luns};
		# now everything else
		while ( my ( $param, $value) = each %$targetdata ) {
			$outfile .= "\t$param $value\n";
		}
	}
print $outfile;	
	open $fh, ">$conffile" or die $text{'error_writefile'} . ':<br />' . $!;
	print $fh $outfile;
	close $fh;
}


sub ietd_status {
-f "/proc/net/iet/session" ? return 1 : return 0
}

sub ietd_start {
	 system($config{'ietstartupscript'}, "start") == 0 or print "<h3> Error : $!</h3>"
}

sub ietd_stop {
	system($config{'ietstartupscript'}, "stop") == 0 or print "<h3> Error : $! </h3>"
}

sub ietd_restart {
	system($config{'ietstartupscript'}, "restart") == 0 or print "<h3> Error : $! </h3>"
}

# end
1;
