######!/usr/bin/perl -w
#

package get_ifaces_info;
use strict;
use Data::Dumper;

# Definitions
my $boot_ifaces_path	= "/etc/network/interfaces";
my $cmd_ifconfig	= test_cmd("ifconfig", "/sbin/ifconfig");

my $not_open_file	= "The file $boot_ifaces_path does not exist or can't be opened.\n";
my $error_ifconfig	= "There was an error during the execution of the command: $cmd_ifconfig.\n";

sub	trans_cidr 
{
    return $_[0] =~ /^255\.255\.255\.(\d+)$/ ? 32-&log2(256-$1) :
	$_[0] =~ /^255\.255\.(\d+)\.0$/ ? 24-&log2(256-$1) :
	$_[0] =~ /^255\.(\d+)\.0\.0$/ ? 16-&log2(256-$1) :
	$_[0] =~ /^(\d+)\.0\.0\.0$/ ? 8-&log2(256-$1) : 32;

}

sub log2
{
    return int(log($_[0])/log(2));
}

sub	get_all_name 
{
    my $if_all;
    
    my @infos = qx!$cmd_ifconfig -a!;
    my $boot_ifc = get_booted_ifaces();

    foreach my $var (@infos) {
	if ( $var =~ /^([\w:]+)\s+/) {
	    $if_all->{$1}= undef ;
	}
    }
    
    foreach my $key (keys %{$boot_ifc})
    {
	$if_all->{$key}= undef ;
    }
    
    return $if_all;
}

sub	get_ifconf_all
{
    my $if_inact;
    my $iface;
    my $mask;
    my $flg = 0;
    my @infos = qx!$cmd_ifconfig -a!;

    foreach my $var (@infos) {
	if ( $var =~ /^([\w:]+)\s+/) {
	    $iface = $1;
	    $if_inact->{$1} = {} if ( ! exists $if_inact->{$1});
	}
	elsif ( $var =~ /^\s+inet ad\w+:([\d\.]*)\s+Bcast:([\d\.]*)\s+Mas\w+:([\d\.]*)\s*/) {
	    $if_inact->{$iface}{'ipinactive'} = $1;	
	    $if_inact->{$iface}{'bcastinactive'} = $2;
	    $if_inact->{$iface}{'maskinactive'} = $3;
	}
    }
    return $if_inact;
}

sub	get_ifconf 
{
    my $if_act;
    my $iface;
    my @infos = qx!$cmd_ifconfig!;

    foreach my $var (@infos) {
	if ( $var =~ /^([\w:]+)\s+/) {
	    $iface = $1;
	    $if_act->{$1} = {} if ( ! exists $if_act->{$1});
	}
	elsif ( $var =~ /^\s+inet ad\w+:([\d\.]*)\s+Bcast:([\d\.]*)\s+Mas\w+:([\d\.]*)\s*/) {
	    $if_act->{$iface}{'ipreal'} = $1;	
	    $if_act->{$iface}{'bcastreal'} = $2 ;	
	    $if_act->{$iface}{'maskreal'} = $3 ;	
	}
    }
    return $if_act;
}

#Recuperer la listes des interfaces et de leurs caracteristiques au boot de la machine
sub	get_booted_ifaces
{
    my $if_boot;
    my $if_name;
    my $tmp;
    
    if ( -f $boot_ifaces_path && -r _ )
    {
	open (FD, $boot_ifaces_path) or die "Error: $!\n";

	while (<FD>) {
	    next if ($_ =~ /^\s*\#/) ;
	    if ($_ =~ /^auto(.*)/){
	        $tmp = $1;
	    }
	    elsif ( $_ =~ /^\s*iface\s+([\w\d:]*)\s+\w*\s+(\w*)/){
		$if_name = $1;
		$if_boot->{$if_name}{'mode'} = $2;
		$if_boot->{$if_name}{'status'} = '0';
	    }
	    elsif ( $_ =~ /\s+address\s+([\d\.]*)/ )
	    {
		$if_boot->{$if_name}{'ipboot'} = $1;
	    }
	    elsif ( $_ =~ /\s+netmask\s+([\d\.]*)/ )
	    {
		$if_boot->{$if_name}{'maskboot'} = $1 ;
	    }
	    elsif ( $_ =~ /\s+broadcast\s+([\d\.]*)/ )
	    {
		$if_boot->{$if_name}{'bcastboot'} = $1 ;
	    }
	}
	foreach (split(" ",$tmp) ){
	    $if_boot->{$_}{'status'} = '1';
	}
	close (FD);
    }
    else{
	print $not_open_file;
	return 1;
    }

    return $if_boot;

}

# Test sur le path et l'existence de la commande ifconfig
sub	test_cmd
{
    my $cmd	= shift;
    my $name	= shift;
    my $func;
    
    chomp($func = `which $cmd`);
    
    if ( ! -e $func){
	$func = $name;
    }

    if ( ! -x $func){
	print "The command $cmd is not executable.\n";
	exit 0;
    }
    return $func;
}

1;
