#!/usr/bin/perl -w
# list_ifcs.cgi
# List active and boot-time interfaces

do '../web-lib.pl';
require './net-lib.pl';

use diagnostics;
use Data::Dumper;
use lib ".";
use get_ifaces_info;

&init_config();
&ReadParse();
&ui_print_header(undef, $text{'index_title'}, "");

my $if_all = get_ifaces_info::get_all_name();
my $if_boot = get_ifaces_info::get_booted_ifaces();
my $if_act = get_ifaces_info::get_ifconf();
my $if_inact = get_ifaces_info::get_ifconf_all();

%{$big_hash} = (
		  'boot' => $if_boot,
		  'act' => $if_act,
		  'inact' => $if_inact,
	       );

my $tmp_hash = Dumper($big_hash);

my $hash_param = uriescape($tmp_hash);

sub uriescape {
    my ($foo) = @_;
    my ($out);
    foreach (split //,$foo)
    {
        if ($_ eq " ") { $out.="+"; next; }
        if((ord($_) >= 0x41 && ord($_) <= 0x5a) ||
           (ord($_) >= 0x61 && ord($_) <= 0x7a) ||
           (ord($_) >= 0x30 && ord($_) <= 0x39))
        {
            $out.=$_ ;
        }
        else {
            $out.=sprintf("%%%02X",ord($_));
        }
    }
  return $out;
}



sub	print_boot {

    my ($ifc, $ip, $mask) = @_;
    my $mask_cidr = get_ifaces_info::trans_cidr($if_boot->{$ifc}{$mask});    
    if ( exists $if_boot->{$ifc}{$ip} && exists $if_boot->{$ifc}{$mask} && ($if_boot->{$ifc}{'status'} eq "1")) {
	print '<td><center>'.$if_boot->{$ifc}{$ip}.'&nbsp;/&nbsp;'.$mask_cidr.'</center></td>';
    }
    elsif (($if_boot->{$ifc}{'mode'} eq "dhcp") && ($if_boot->{$ifc}{'status'} eq "1")) {
	print "<td><center>$if_boot->{$ifc}{'mode'}</center></td>";
    }
    elsif ($if_boot->{$ifc}{'mode'} eq "loopback") {
	print "<td><center>$if_boot->{$ifc}{'mode'}</center></td>";
    }
   else {
	print '<td>&nbsp;</td>';
   }
}

sub	print_real {

    my ($ifc, $ip, $mask) = @_;

    if ( exists $if_act->{$ifc}{$ip} && exists $if_act->{$ifc}{$mask}) {
	my $mask_cidr = get_ifaces_info::trans_cidr($if_act->{$ifc}{$mask});
	print '<td><center>'.$if_act->{$ifc}{$ip}.'&nbsp;/&nbsp;'.$mask_cidr.'</center></td>';
    }
    else {
	print '<td>&nbsp;</td>';
    }

}

sub	print_inactive {
    my ($ifc, $ip, $mask) = @_;
    my $mask_cidr = get_ifaces_info::trans_cidr($if_act->{$ifc}{$mask});
    if ( exists $if_inact->{$ifc}{$ip} && exists $if_inact->{$ifc}{$mask} && ( ! exists $if_act->{$ifc} ) ) {
	print '<td><center>'.$if_inact->{$ifc}{$ip}.'&nbsp;/&nbsp;'.$mask_cidr.'</center></td>';
    }
    else {
	print '<td>&nbsp;</td>';
    }
    
}

sub	print_mode {
    my ($ifc, $ip) = @_;
    
    if ( exists $if_boot->{$ifc}{$ip}) {
	print '<td><center>'.$if_boot->{$ifc}{$ip}.'</center></td>';
    }
    else {
	print '<td>&nbsp;</td>';
    }
    
}

if ( defined $if_all) {
    print <<EOF;
    <br><br>
    <form methode="get" action="ifc_config.cgi">
	<table border=2 align=center width="100%">
	<tr bgcolor="#9999ff">
	<td align=center width="20%"><b>$text{'ifcs'}</b></td>
	<td align=center width="20%"><b>$text{'ipboot'}</b></td>
	<td align=center width="20%"><b>$text{'ipreal'}</b></td>
	<td align=center width="20%"><b>$text{'ipinactive'}</b></td>
	<td align=center width="20%"><b>$text{'ifcs_mode'}</b></td>
	</tr>
	
EOF

foreach my $iface (sort keys (%{$if_all})) {
	print "<tr bgcolor=#cccccc><td><a href=ifc_config.cgi?param=$iface&amp;param1=$hash_param>$iface</a></td>";
    
	print_boot($iface, "ipboot", "maskboot");
	print_real($iface, "ipreal", "maskreal");
	print_inactive($iface, "ipinactive", "maskinactive");
	print_mode($iface, "mode");
}

    print <<EOF;;
    </tr></table></center></form><br>
    <a href="ifc_new.cgi">$text{'add_ifc'}</a><br>
    
EOF

}

&ui_print_footer("", $text{'ifcs_return'});
