#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## ifc_config.pl for Configuration Interfaces in 
## 
## ######### PROJECT DESCRIPTION : ###
## 
## 
## 
## ###################################
## 
## Made by Mickael
## Login   <mike@inguera.com>
## 
## Started on  Mon May 28 16:07:50 2007 Mickael
## 
## ###################################
##

do '../web-lib.pl';
require './net-lib.pl';

use diagnostics;
use Data::Dumper;
use warnings;

&init_config();
&ReadParse();
&ui_print_header(undef, $text{'edit_ifc'}, "");

my $config		= "/etc/network/interfaces";
my $ifconfig		= test_cmd("ifconfig", "/sbin/ifconfig");
my $pump		= test_cmd("pump", "/sbin/pump");
my $op_failed		= "Operation Failed.<br>";
my $op_done		= "Operation succeeded.<br>";

my $param = {
    'address'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
    'netmask'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
    'broadcast'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
    'network'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
    'gateway'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
};

sub     test_cmd
{
    my $cmd     = shift;
    my $name    = shift;
    my $func;

    chomp($func = `which $cmd`);

    if ( ! -e $func){
        $func = $name;
    }

    if ( ! -x $func){
        print "The command $cmd is not executable.\n";
        exit 0;
    }
    return $func;
}

sub	cmd_exec
{
    my $cmd_line = shift;
    
    eval {
	qx!$cmd_line!;
    };
    if ($?) {
	return 42;
    }
    
    return 0;
}

sub	read_conf_file
{
    my $info;
    my $name;
    my $iface = $data->{'interface'};

    if ( -f $config && -r _ ) {
	open(FD_READ, "< $config") or die "Error: $!.<br>";
	my @tab = <FD_READ>;
	close(FD_READ);
	
	foreach my $line (@tab){
	    if ( $line =~ /^(\#.*)/) {
		$info->{'commentaires'} .= $1."\n";
		next;
	    }
	    if ($line =~ /^auto\s+(.*)$/){
		$info->{'auto'} = $1;
		next;
	    }
	    if ($line =~ /^iface\s+([\w+\d:]+)\s+inet\d?\s+(\w+)/){
		$name = $1;
		$info->{'inter'}{$name}{'mode'} = $2;
	    }
	    else {
		if ( $line =~ /^\s*(\w+)\s+(.*)/){
		    my $label = $1;
		    my $value = $2;
		    if ( exists $param->{$label} and ($value =~ /$param->{$label}/) )
		    {
			$info->{'inter'}{$name}{$label} = $value;
		    }
		}
		elsif ( $line =~ /^\s*(\#.*)/) {
		    $info->{'inter'}{$name}{'commentaires'} = $1;
		}
	    }
	}
    }

    return $info;
}

sub	write_conf_file
{
    my $hash_conf = shift;

    open(FD, "> $config") or die "Error: $!.<br>";    

    print FD $hash_conf->{'commentaires'}."\n" if ( exists $hash_conf->{'commentaires'} );
  
    print FD "auto ".$hash_conf->{'auto'}."\n\n";
    
    if (exists $hash_conf->{'inter'}{'lo'} and $hash_conf->{'inter'}{'lo'}{'mode'}) {
	print FD "iface lo inet ".$hash_conf->{'inter'}{'lo'}{'mode'}."\n\n";
	delete $hash_conf->{'inter'}{'lo'};
    }

    foreach my $key (sort  keys %{$hash_conf->{'inter'}}) {
	print FD "iface $key inet ".$hash_conf->{'inter'}{$key}{'mode'}."\n";
	print FD "\taddress ".$hash_conf->{'inter'}{$key}{'address'}."\n"	if ( $hash_conf->{'inter'}{$key}{'address'} );
	print FD "\tbroadcast ".$hash_conf->{'inter'}{$key}{'broadcast'}."\n"	if ( $hash_conf->{'inter'}{$key}{'broadcast'} );
	print FD "\tnetmask ".$hash_conf->{'inter'}{$key}{'netmask'}."\n"	if ( $hash_conf->{'inter'}{$key}{'netmask'} );
	print FD "\tnetwork ".$hash_conf->{'inter'}{$key}{'network'}."\n"	if ( $hash_conf->{'inter'}{$key}{'network'} );
	print FD "\tgateway ".$hash_conf->{'inter'}{$key}{'gateway'}."\n"	if ( $hash_conf->{'inter'}{$key}{'gateway'} );
	print FD "\t".$hash_conf->{'inter'}{$key}{'commentaires'}."\n"	if ( $hash_conf->{'inter'}{$key}{'commentaires'} );
	print FD "\n";
    }
    close(FD);
}

sub	apply_now
{
    my $hash = shift ;

    if ( $hash->{'flagip'} eq "auto" ) {
	if ( (cmd_exec("$pump -i $hash->{'interface'}")) ) {
	    print $op_failed;
	}
	else{
	    print $op_done;
	}
    }
    else {
	if ( ($hash->{'ip'} eq "") || ($hash->{'mask'} eq "" && $hash->{'flagmask'} eq "manu") || ($hash->{'broad'} eq ""  && $hash->{'broadmask'} eq "manu")){
 	    print "$text{'error_param'}<br>";
	    return 0;
  	}
	if ( $hash->{'flagmask'} eq "auto" ) {
	    $hash->{'mask'} = "255.255.255.0";
	}
	if ( $hash->{'flagbroad'} eq "auto" ) {
	    if ( $hash->{'ip'} =~ /(^\d{1,3}\.\d{1,3}\.\d{1,3})\.\d+/)
	    {
		my $tmp = $1.".255";
		$hash->{'broad'} = $tmp;
	    }
	}

	if ((cmd_exec("$ifconfig $hash->{'interface'} $hash->{'ip'} netmask $hash->{'mask'} broadcast $hash->{'broad'}" )))
	{
	    print $op_failed;
	}
	else{
	    print $op_done;
	}
    }    
}

sub	apply_boot
{
    my $hash = shift ;
    my $hash_conf_file = read_conf_file();

    if ( $hash_conf_file->{'auto'} !~ $hash->{'interface'} ) {
	$hash_conf_file->{'auto'} .= " $hash->{'interface'}";
    }

    if ( $hash->{'flagip'} eq "auto") {
	delete $hash_conf_file->{'inter'}{$hash->{'interface'}} if exists $hash_conf_file->{'inter'}{$hash->{'interface'}};
	$hash_conf_file->{'inter'}{$hash->{'interface'}}{'mode'} = "dhcp";
    }
    else {
	if ( ($hash->{'ip'} eq "") || ($hash->{'mask'} eq "" && $hash->{'flagmask'} eq "manu") || ($hash->{'broad'} eq ""  && $hash->{'broadmask'} eq "manu")){
 	    print "$text{'error_param'}<br>";
 	    return 0;
	}
	if ( $hash->{'flagmask'} eq "auto" ) {
	    $hash->{'mask'} = "255.255.255.0";
	}
	if ( $hash->{'flagbroad'} eq "auto" ) {
	    if ( $hash->{'ip'} =~ /(^\d{1,3}\.\d{1,3}\.\d{1,3})\.\d+/)
	    {
		my $tmp = $1.".255";
		$hash->{'broad'} = $tmp;
	    }
	}
 	$hash_conf_file->{'inter'}{$hash->{'interface'}}{'address'}	= $hash->{'ip'} ;
 	$hash_conf_file->{'inter'}{$hash->{'interface'}}{'netmask'}	= $hash->{'mask'};
 	$hash_conf_file->{'inter'}{$hash->{'interface'}}{'broadcast'}	= $hash->{'broad'};
 	$hash_conf_file->{'inter'}{$hash->{'interface'}}{'mode'}	= "static";
# 	$hash_conf_file->{'inter'}{$hash->{'interface'}}{'network'}	= "voir sylvain";
    }

    write_conf_file($hash_conf_file);
    print $op_done;
}

sub	disable_now
{
    my $hash = shift ;

    if ((cmd_exec("$ifconfig $hash->{'interface'} down" )))
    {
	print $op_failed;
    }
    else{
	print $op_done;
    }
}

sub	disable_boot
{
    my $hash = shift ;
    
    my $hash_conf_file = read_conf_file();

    $hash_conf_file->{'auto'} =~ s/$hash->{'interface'}// ;
    
    write_conf_file($hash_conf_file);
    
    print $op_done;
    
}

sub	delete_int
{
    my $hash = shift;

    disable_boot($hash);
    my $hash_conf_file = read_conf_file();
    
    delete $hash_conf_file->{'inter'}{$hash->{'interface'}};
    
    write_conf_file($hash_conf_file);
    
}

my $info = {
    interface =>$in{'iface'}, 
    flagip => $in{'ip'}, 
    flagmask => $in{'mask'},
    flagbroad => $in{'broad'}, 
    ip => $in{'ifc_ip'},
    mask => $in{'ifc_mask'},
    broad => $in{'ifc_broad'},
};

if ( exists $in{'an'}) {	
    apply_now($info);
}
elsif (exists $in{'ab'}) {
    apply_boot($info);
}
elsif (exists $in{'anab'}) {
    apply_boot($info);
    apply_now($info);
}
elsif (exists $in{'dn'}) {
    disable_now($info);
}
elsif (exists $in{'db'}) {
    disable_boot($info);
}
elsif (exists $in{'dnab'}) {
    disable_boot($info);
    disable_now($info);
}
elsif (exists $in{'del'}) {
    delete_int($info);
}

&ui_print_footer("list_ifcs.cgi", $text{'ifcs_return'});

