#! /usr/bin/perl
# index.cgi
# Display all RAID controllers, logical units and ports

do '../web-lib.pl';

use strict;
use warnings;

use Lib_Raid_Rpc;
use Lib_Raid::lib_raid_codes;
use Data::Dumper;

our ( %in, %text );

&init_config();
&ReadParse();

&header($text{'view_device'}, "", undef, 0, 1, 0);

########################
# functions

sub display_lun_head {
    print "<tr bgcolor=\"#cccccc\">
            <td>UNIT</td>
            <td>Status</td>
            <td>rebuild</td>
            <td>Size</td>
            <td>Array</td>
        </tr>";
}

sub display_array_head {
    print "<tr bgcolor=\"#cccccc\">
            <td>UNIT</td>
            <td>RAID level</td>
            <td>Status</td>
            <td>rebuild</td>
            <td>Size</td>
        </tr>";
}

sub display_disks_head {
    print "<tr bgcolor=\"#cccccc\">
            <td>Port</td>
            <td>Status</td>
            <td>Unit</td>
            <td>Size</td>
            <td>Model</td>
            <td>Serial Number</td>
        </tr>";
}

sub display_disk {

    my $port = shift;
    my $disk = shift;

    my $color = "#dd3311";
    if ( $disk->{status} =~ /OK/ ) {
        $color = "#ffffff";
    }

    my $disk_status = lib_raid_codes::get_drive_state_string(
        $disk->{status} );

    print "<tr>
            <td>$port</td>
            <td>$disk_status</td>
            <td>$disk->{inarray}</td>
            <td>$disk->{size}&nbsp;GB</td>
            <td>$disk->{model}</td>
            <td>$disk->{serialnumber}</td>
        </tr>";

}

########################
# controller details
if ( exists $in{ctldet} ) {

    # début du tableau HTML
    print "<table border=\"1\">
            <tr bgcolor=\"#999\">
                <th colspan=\"8\">$text{index_ctlinfo}&nbsp;$in{ctldet}</th>
            </tr>
            <tr bgcolor=\"#999\">
                <th colspan=\"8\">$text{index_return}</th>
            </tr>";

    display_lun_head();

    my $ctl     = $in{ctldet};
    my $ctl_ref = Lib_Raid_Rpc::get_all_info($ctl);
	
    if ( exists $ctl_ref->{$ctl} and defined $ctl_ref->{$ctl} ) {
        my $controller = $ctl_ref->{$ctl};
        foreach my $lun_k ( sort keys %{ $controller->{luns} } ) {
            my $lun_v = $controller->{luns}->{$lun_k};

            my $array = join( ' ', @{ $lun_v->{arrays} } );
            print "<tr>
                    <td>$lun_k</td>
                    <td>$lun_v->{status}</td><td />
                    <td>$lun_v->{size}&nbsp;GB</td>
                    <td>$array</td>
                </tr><tr bgcolor=\"#cccccc\">
                    <td>Array</td>
                    <td>Status</td>
                    <td>Rebuild</td>
                    <td>Size</td>
                    <td>Disks</td>
                </tr>";

            foreach $array ( sort @{ $lun_v->{arrays} } ) {
                my $array_ref = $controller->{arrays}->{$array};
                my $disks = join ' ', @{ $array_ref->{drives} };
                print "<tr>
                        <td>$array</td>
                        <td>$array_ref->{status}</td>
						<td>$array_ref->{progression}</td>
                        <td>$array_ref->{size}&nbsp;MB</td>
                        <td>$disks</td>
                       </tr>";

			print
            "</tr><tr bgcolor=\"#999\"><th colspan=\"8\">$text{index_ports}</th></tr>";
			
                display_disks_head();
                foreach my $disk ( @{ $array_ref->{drives} } ) {
                    my $disk_ref    = $controller->{drives}->{$disk};
					display_disk($disk, $disk_ref);
                }
            }

            my @other_disks;
            foreach my $disk ( sort keys %{ $controller->{drives} } ) {
                push @other_disks, $disk
                    if ( $controller->{drives}->{$disk}->{inarray} < 0 );
            }

            if ( @other_disks > 0 ) {
                print "<tr bgcolor=\"#cccccc\">
						<td colspan=\"6\">
							<center>Other disks</center>
						</td>
					</tr>
					";

                foreach my $disk (@other_disks) {
                    my $disk_ref    = $controller->{drives}->{$disk};
					
					display_disk($disk, $disk_ref);
                }
            }
        }
        
        # fin du tableau général
        print "</table>";

    &footer( "./index.cgi", $text{home} );
    }

#     print "</table><br /><pre>controller [$ctl]<br />", Dumper($ctl_ref),
#         '<pre />';

    exit;
}

########################
# array details
if ( exists $in{voldet} && exists $in{ctl} ) {

    print
        "<table border=\"1\"><tr bgcolor=\"#999\"><th colspan=\"9\">$text{index_return_info}&nbsp;cont.&nbsp;$in{ctl}&nbsp;unit&nbsp;$in{voldet}</th></tr>";

    display_array_head();

    my $ctl_ref = Lib_Raid_Rpc::get_all_info();
	
	if ( exists $ctl_ref->{ $in{ctl} } and defined $ctl_ref->{ $in{ctl} } and defined $ctl_ref->{ $in{ctl} }{arrays}{ $in{voldet}} ){

	 my $array_ref = $ctl_ref->{$in{ctl} }{arrays}{ $in{voldet}};
	 
	 my $array_status = lib_raid_codes::get_drive_state_string(
                        $array_ref->{status} );
		print "<tr>
		        <td> $in{voldet} </td>
				<td> $array_ref->{raidtype} </td>
				<td> $array_status </td>
				<td> $array_ref->{progression} </td>
				<td> $array_ref->{size} MB </td>
			  </tr>
	";

    print "</tr>
        <tr bgcolor=\"#999\"><th colspan=\"8\">
        $text{index_ports}</th></tr>";

    display_disks_head();

                foreach my $disk ( @{ $array_ref->{drives} } ) {
                    my $disk_ref    = $ctl_ref->{ $in{ctl} }{drives}->{$disk};
					
					display_disk($disk, $disk_ref);	

                }


	}
	

    print "</table>";
    &footer( "./index.cgi", $text{home} );
    exit;
}

