#!/usr/bin/perl -w 

do '../web-lib.pl';
use strict;
use warnings;
use Lib_Raid_Rpc;


&init_config();
&ReadParse();

our %text;

&header($text{'index_title'}, "");

print <<EOF;

    <table style="align: center; cellspacing:5;">
    <tr>
    <td align=center><a href="do_raid.cgi"><img src="icons/raid_add_64x64.png"></a></td>
    <td align=center><a href="do_delete.cgi"><img src="icons/raid_delete_64x64.png"></a></td>
    <td align=center><a href="do_rescan.cgi"><img src="icons/raid_rescan_64x64.png"></a></td>
    <td align=center><a href="do_rebuild.cgi"><img src="icons/raid_rebuild_64x64.png"></a></td>
    </tr>
    <tr>
    <td align=center><a href="do_raid.cgi">$text{'create_raid'}</a></td>
    <td align=center><a href="do_delete.cgi">$text{'delete_raid'}</a></td>
    <td align=center><a href="do_rescan.cgi">$text{'rescan_raid'}</a></td>
    <td align=center><a href="do_rebuild.cgi">$text{'verify_raid'}</a></td>
    </tr>
    </table>

EOF
;
    

my $raid_type = {
    -2 => 'JBOD',
    -1 => 'SINGLE',
    0  => 'RAID 0',
    1  => 'RAID 1',
    5  => 'RAID 5',
    6  => 'RAID 6',
    10 => 'RAID 10',
    50 => 'RAID 50',
    60 => 'RAID 60',
};

    &header( "", "", undef, 0, 1, 0 );

sub _get_correct_size {
    my $size = shift;

    return $size if ( $size !~ m/[\d\.]/ );

    if ( $size > 1000000 ) {
        $size = sprintf( "%.2f", $size / ( 1024 * 1024 ) ) . " TB";
    } elsif ( $size > 1000 ) {
        $size = sprintf( "%.2f", $size / 1024 ) . " GB";
    } else {
        $size = sprintf( "%.2f", $size ) . " MB";
    }
    return $size;
}

sub unit_icon {

    # renvoie l'icône
    my ( $unittype, $status ) = @_;

    $status = 'OTHER' unless $status eq 'OK';

    my %translate = (
        JBOD => {
            OK    => 'images/raid_0_bg.png',
            OTHER => 'images/raid_0_fail_bg.png',
        },
        SPARE => {
            OK    => 'images/raid_spare_bg.png',
            OTHER => 'images/raid_spare__failed_bg.png',
        },
        SINGLE => {
            OK    => 'images/raid_single_bg.png',
            OTHER => 'images/raid_single_fail_bg.png',
        },
        'RAID 0' => {
            OK    => 'images/raid_0_bg.png',
            OTHER => 'images/raid_0_bg.png',
        },
        'RAID 1' => {
            OK    => 'images/raid_1_bg.png',
            OTHER => 'images/raid_1_fail_bg.png',
        },
        'RAID 5' => {
            OK    => 'images/raid_5_bg.png',
            OTHER => 'images/raid_5_fail_bg.png',
        },
        'RAID 6' => {
            OK    => 'images/raid_6_bg.png',
            OTHER => 'images/raid_6_fail_bg.png',
        },
        'RAID 10' => {
            OK    => 'images/raid_10_bg.png',
            OTHER => 'images/raid_10_fail_bg.png',
        },
        'RAID 01' => {
            OK    => 'images/raid_10_bg.png',
            OTHER => 'images/raid_10_fail_bg.png',
        },
        'RAID 50' => {
            OK    => 'images/raid_50_bg.png',
            OTHER => 'images/raid_50_fail_bg.png',
        },
        'RAID 05' => {
            OK    => 'images/raid_50_bg.png',
            OTHER => 'images/raid_50_fail_bg.png',
        },
        'RAID 60' => {
            OK    => 'images/raid_60_bg.png',
            OTHER => 'images/raid_60_fail_bg.png',
        },
        'RAID 05' => {
            OK    => 'images/raid_50_bg.png',
            OTHER => 'images/raid_50_fail_bg.png',
        },
    );

    return $translate{$unittype}{$status};
}

# Affiche le menu icones
sub print_ctl_infos {
    my $controllers_ref = Lib_Raid_Rpc::get_all_info();

    if ($controllers_ref) {
        all_infos($controllers_ref);
    } else {
        print $text{index_none};
    }
}

sub all_infos {
    my %treeicons = (
        vertical   => 'images/vertical.png',
        horizontal => 'images/horizontal.png',
        angle      => 'images/angle.png',
        treesplit  => 'images/treesplit.png',
        space      => 'images/tree_space.gif'
    );

    # DISPLAY  controllers and basic status

    my $ctls_ref = shift;
    ########################
    # affichage

    print "<br /><br />";

    print qq(
	<table style="border:0; width:100%; cellspacing:0; padding:0;">
        <tr bgcolor="#999">
		 <th colspan="2">$text{index_ctl}</th>
		 <th colspan="2">$text{index_vol}</th>
		 <th colspan="2">$text{index_ports}</th>
		 </tr>\n);

    foreach my $controller ( sort keys %$ctls_ref ) {

        my $controllerdata = $ctls_ref->{$controller};
        my $controllericon = 'images/card.png';

        print qq(<tr>
	        <td style="padding: 0; width: 64px; height: 64px;"><img src="$controllericon"></td>
			<td style="padding: 0; width: 192px; height: 64px;"><a href="details.cgi?ctldet=$controller" title="$text{ctlinfo}"> $controller </a></td>
			<td colspan="4">&nbsp;</td>
		   </tr>);

        my $arraycount = 1;
        my $arraylast  = $controllerdata->{numberofarrays};

        foreach my $array ( sort keys %{ $controllerdata->{arrays} } ) {
            my $arraydata    = $controllerdata->{arrays}->{$array};
            my $arraysize    = _get_correct_size( $arraydata->{size} );
            my $treeicon     = $treeicons{treesplit};
            my $verticalicon = $treeicons{vertical};
            my $disksingle   = $treeicons{space};

            my $arrayicon = unit_icon(
                $raid_type->{ $arraydata->{raidtype} },
                $arraydata->{status} == 0 ? 'OK' : 'OTHER'
            );

            if ( ++$arraycount > $arraylast ) {
                $treeicon     = $treeicons{angle};
                $verticalicon = $treeicons{space};
            }

            print qq(<tr>
   	        <td  style="padding: 0; width: 64px; height: 64px;"><img src="$treeicon"  style="padding: 0; width: 64px; height: 64px;"></td>
   			<td style="padding: 0; width: 192px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"></td>
   			<td  style="padding: 0; width: 64px; height: 64px;"><img src="$arrayicon"></td>
   	        <td style="padding: 0; width: 64px; height: 64px;"><a href="details.cgi?voldet=$array&ctl=$controller" title="$text{ctlinfo}">$array ($arraysize)</a></td>
   			<td colspan="2">&nbsp;</td>
            </tr>);

            my $diskcount = 1;
            my $disklast  = @{ $arraydata->{drives} };
			
			# displays drives part of some array
            foreach my $disk (
                sort {
                         ( $a =~ /(\D)*\d*/ )[0] cmp( $b =~ /(\D)*\d*/ )[0]
                      || ( $a =~ /[\D]*(\d*)/ )[0] <=> ( $b =~ /[\D]*(\d*)/ )[0]
                } @{ $arraydata->{drives} }
              )
            {
                my $diskdata     = $controllerdata->{drives}->{$disk};
                my $diskicon     = 'images/hdd.png';
                my $disktreeicon = $treeicons{treesplit};
                my $disksize     = _get_correct_size( $diskdata->{size} );

                $diskicon = 'images/hdd_bad.png'
                  unless $diskdata->{status} == 0;
                if ( ++$diskcount > $disklast ) {
                    $disktreeicon = $treeicons{angle};
                }

                print qq(<tr>
   		        <td style="padding: 0; width: 64px; height: 64px;"><img src="$verticalicon" style="padding: 0; width: 64px; height: 64px;"></td>
   				<td style="padding: 0; width: 192px; height: 64px;">
                    <img src="$disksingle" style="padding: 0; width: 192px; height: 64px;">
                </td>
   		        <td style="padding: 0; width: 64px; height: 64px;"><img src="$disktreeicon" style="padding: 0; width: 64px; height: 64px;"></td>
   				<td  style="padding: 0; width: 192px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"></td>
   		        <td style="padding: 0; width: 64px; height: 64px;"><img src="$diskicon"></td>
   				<td><a href="details.cgi?voldet=$array&ctl=$controller" title="$text{ctlinfo}">$disk ($disksize)</a></td>
   			   </tr>);
            }
			
			# displays enclosures
			my $enc_icon='images/enclosure.png';
			
			if ( exists( $controllerdata->{enclosures} )) {
				# PAS FINI
				foreach my $enc ( sort keys %{ $controllerdata->{enclosures} } ) {
					            print qq(<tr>
   	        <td style="padding: 0; width: 64px; height: 64px;"><img src="$treeicon"  style="padding: 0; width: 64px; height: 64px;"></td>
   			<td  style="padding: 0; width: 192px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"></td>
   			<td  style="padding: 0; width: 64px; height: 64px;"><img src="$enc_icon"></td>
   	        <td><a href="details.cgi?encdet=$enc&ctl=$controller" title="$text{ctlinfo}">$enc</a></td>
   			<td colspan="2">&nbsp;</td>
            </tr>);

				}
			}
			
			
			# display hot spares
			my $hsp_icon='images/raid_spare_bg.png';
               $treeicon     = $treeicons{angle};
               $verticalicon = $treeicons{vertical};
               $disksingle   = $treeicons{space};
			my $hsp_code = -2 ;
			
 			foreach my $disk ( keys ( %{$controllerdata->{drives}} ) ) {
 				
				next if ( $controllerdata->{drives}{$disk}{inarray} != $hsp_code );
				
				$disk =~ tr/d/h/;
				
				# actually display it
				print qq(<tr>
				<td style="padding: 0; width: 64px; height: 64px;"><img src="$treeicon"  style="padding: 0; width: 64px; height: 64px;"></td>
   			<td  style="padding: 0; width: 192px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"></td>
   			<td  style="padding: 0; width: 64px; height: 64px;"><img src="$treeicons{horizontal}"></td>
   	        <td  style="padding: 0; width: 192px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"><img src="$treeicons{horizontal}"  style="padding: 0; width: 64px; height: 64px;"></td>
   			<td  style="padding: 0; width: 64px; height: 64px;"><img src="$hsp_icon"></td>
   	        <td><a href="details.cgi?diskdet=$disk&ctl=$controller" title="$text{ctlinfo}">$disk</a></td>
   			<td colspan="2">&nbsp;</td>
            </tr>);
 			}
 			
			# display dangling disks
			
			
        }
    }

    print "</table>";

}

print_ctl_infos();


&footer("/", $text{'index'});

