# export-lib.pl
# Common functions for the linux exports file

do '../web-lib.pl';
&init_config();
require '../ui-lib.pl';

use Data::Dumper;
require Objet_Conf;

# recupere la hash de conf d'un fichier
sub getconfig {
    my $path = shift;
    my %hash;
    my $conf = new Objet_Conf($path);
    %hash = $conf->get_all();

    return %{ $hash{DEFAUTINTELLIQUEUNIQUE} };
}


# Retrouve le mode d authentification
sub authenfication_mode {
    my ( $pam_conf, $guest_enable ) = @_;
    open(FILE, $pam_conf ) or return "vide";
    while (<FILE>) 
    {
		next				if ( m/^\s*#/ );
		return "virtual"	if ( m/auth\s*required[0-9a-zA-Z_\/\.\- ]*db=/ );
		return "openldap"	if ( m/auth\s*sufficient[0-9a-zA-Z_\/\.\- ]*pam_ldap.so/ );
    }
	close FILE;
	return "local_guest"	if ( $guest_enable =~ m/yes/i );
	return "local";
}


#Fonction boolean lecture(0) / ecriture(1)
sub get_readonly {
    my ( $authe,
		$bool_guest_enable,
		$download_enable,
		$write_enable,
		$anon_mkdir_write_enable,
		$anon_upload_enable,
		$anon_other_write_enable,
		$anon_world_readable_only ) = @_;
 
    return 0 if ($write_enable =~ m/no/i);
	
	if ( $authe eq 'local' ) {
	    if ( $write_enable =~ m/yes/i && $download_enable =~ m/yes/i) {
			return 1;
		}
	}
	elsif ( $write_enable =~ m/yes/i &&
			 $download_enable =~ m/yes/i &&
			 $anon_mkdir_write_enable =~ m/yes/i && 
			 $anon_upload_enable =~ m/yes/i && 
			 $anon_other_write_enable =~ m/yes/i &&
			 $anon_world_readable_only =~ m/no/i ) {		 
		return 1;
    }

    return 0;
}

# Met a jour /etc/vsftpd.conf
sub save_conf {
	my $vsftp_conf  = shift;
	my $hash        = shift;
	my $conf = new Objet_Conf($vsftp_conf);

	my $ret = 0;
	foreach ( keys %{ $hash } ) {
	    my ($ret_value) = $conf->set_value($_, $hash{$_});
    	    $ret = 1 if ($ret_value);
	}

	$conf->delete_key("local_root") if ($hash{chroot_local_user} =~ /no/i);
	$conf->delete_key("local_root") if (!(exists $hash{local_root}));
	$conf->delete_key("guest_username") if (!(exists $hash{guest_username}));
	$conf->save();

	open(FILE, $vsftp_conf );
    my @file = <FILE>;
    close FILE;
	open(FILE, "> $vsftp_conf" );
	foreach ( @file ) {
    	$_ =~ s/\s*=\s*/=/g;
        print FILE $_;
	}
	close FILE;

	return $ret;
}

#Fonction boolean lecture(0) / ecriture(1)
sub set_readonly {
	my $readonly = shift;

	# Ecriture
	return ( "yes", "yes", "yes", "yes", "yes", "no") if ( $readonly =~ m/yes/i );

	# Lecture seule
	return ( "yes", "no", "no", "no", "no", "yes");
}

#Fonction boolean NO(0) / YES(1)
sub get_boolean_value {
	my $val = shift;

	return 1 if ( $val =~ /yes/i );
	return 0;
}

sub	set_conf {
    my ($file, $key, $value)	= @_;
    my $conf = new Objet_Conf($file);
    $conf->set_value($key, $value);
    $conf->save();
}

1;
